@@ -47,7 +47,11 @@ class CommandCluster : public Commander {
 
     if (subcommand_ == "import") {
       if (args.size() != 4) return {Status::RedisParseErr, errWrongNumOfArguments};
-      slot_ = GET_OR_RET(ParseInt<int64_t>(args[2], 10));
+
+      Status s = CommandTable::ParseSlotRanges(args_[2], slot_ranges_);
+      if (!s.IsOK()) {
+        return s;
+      }
 
       auto state = ParseInt<unsigned>(args[3], {kImportStart, kImportNone}, 10);
       if (!state) return {Status::NotOK, "Invalid import state"};
@@ -109,7 +113,8 @@ class CommandCluster : public Commander {
         return s;
       }
     } else if (subcommand_ == "import") {
-      Status s = srv->cluster->ImportSlot(conn, static_cast<int>(slot_), state_);
+      // TODO: support multiple slot ranges
+      Status s = srv->cluster->ImportSlotRange(conn, slot_ranges_[0], state_);
       if (s.IsOK()) {
         *output = redis::SimpleString("OK");
       } else {
@@ -138,7 +143,7 @@ class CommandCluster : public Commander {
 
  private:
   std::string subcommand_;
-  int64_t slot_ = -1;
+  std::vector<SlotRange> slot_ranges_;
   ImportStatus state_ = kImportNone;
 };
 
@@ -154,7 +159,10 @@ class CommandClusterX : public Commander {
     if (subcommand_ == "migrate") {
       if (args.size() < 4 || args.size() > 6) return {Status::RedisParseErr, errWrongNumOfArguments};
 
-      slot_ = GET_OR_RET(ParseInt<int64_t>(args[2], 10));
+      Status s = CommandTable::ParseSlotRanges(args_[2], slot_ranges_);
+      if (!s.IsOK()) {
+        return s;
+      }
 
       dst_node_id_ = args[3];
 
@@ -279,8 +287,8 @@ class CommandClusterX : public Commander {
       if (sync_migrate_) {
         sync_migrate_ctx_ = std::make_unique<SyncMigrateContext>(srv, conn, sync_migrate_timeout_);
       }
-
-      Status s = srv->cluster->MigrateSlot(static_cast<int>(slot_), dst_node_id_, sync_migrate_ctx_.get());
+      // TODO: support multiple slot ranges
+      Status s = srv->cluster->MigrateSlotRange(slot_ranges_[0], dst_node_id_, sync_migrate_ctx_.get());
       if (s.IsOK()) {
         if (sync_migrate_) {
           return {Status::BlockingCmd};
@@ -303,7 +311,6 @@ class CommandClusterX : public Commander {
   std::string nodes_str_;
   std::string dst_node_id_;
   int64_t set_version_ = 0;
-  int64_t slot_ = -1;
   std::vector<SlotRange> slot_ranges_;
   bool force_ = false;
 
@@ -322,7 +329,7 @@ static uint64_t GenerateClusterFlag(uint64_t flags, const std::vector<std::strin
 
 class CommandReadOnly : public Commander {
  public:
-  Status Execute(Server *srv, Connection *conn, std::string *output) override {
+  Status Execute([[maybe_unused]] Server *srv, Connection *conn, std::string *output) override {
     *output = redis::SimpleString("OK");
     conn->EnableFlag(redis::Connection::kReadOnly);
     return Status::OK();
@@ -331,7 +338,7 @@ class CommandReadOnly : public Commander {
 
 class CommandReadWrite : public Commander {
  public:
-  Status Execute(Server *srv, Connection *conn, std::string *output) override {
+  Status Execute([[maybe_unused]] Server *srv, Connection *conn, std::string *output) override {
     *output = redis::SimpleString("OK");
     conn->DisableFlag(redis::Connection::kReadOnly);
     return Status::OK();
@@ -340,14 +347,15 @@ class CommandReadWrite : public Commander {
 
 class CommandAsking : public Commander {
  public:
-  Status Execute(Server *srv, Connection *conn, std::string *output) override {
+  Status Execute([[maybe_unused]] Server *srv, Connection *conn, std::string *output) override {
     conn->EnableFlag(redis::Connection::kAsking);
     *output = redis::SimpleString("OK");
     return Status::OK();
   }
 };
 
-REDIS_REGISTER_COMMANDS(MakeCmdAttr<CommandCluster>("cluster", -2, "cluster no-script", 0, 0, 0, GenerateClusterFlag),
+REDIS_REGISTER_COMMANDS(Cluster,
+                        MakeCmdAttr<CommandCluster>("cluster", -2, "cluster no-script", 0, 0, 0, GenerateClusterFlag),
                         MakeCmdAttr<CommandClusterX>("clusterx", -2, "cluster no-script", 0, 0, 0, GenerateClusterFlag),
                         MakeCmdAttr<CommandReadOnly>("readonly", 1, "cluster no-multi", 0, 0, 0),
                         MakeCmdAttr<CommandReadWrite>("readwrite", 1, "cluster no-multi", 0, 0, 0),
