@@ -42,10 +42,11 @@ class CommandPush : public Commander {
     uint64_t ret = 0;
     rocksdb::Status s;
     redis::List list_db(srv->storage, conn->GetNamespace());
+    engine::Context ctx(srv->storage);
     if (create_if_missing_) {
-      s = list_db.Push(args_[1], elems, left_, &ret);
+      s = list_db.Push(ctx, args_[1], elems, left_, &ret);
     } else {
-      s = list_db.PushX(args_[1], elems, left_, &ret);
+      s = list_db.PushX(ctx, args_[1], elems, left_, &ret);
     }
     if (!s.ok()) {
       return {Status::RedisExecErr, s.ToString()};
@@ -111,9 +112,10 @@ class CommandPop : public Commander {
 
   Status Execute(Server *srv, Connection *conn, std::string *output) override {
     redis::List list_db(srv->storage, conn->GetNamespace());
+    engine::Context ctx(srv->storage);
     if (with_count_) {
       std::vector<std::string> elems;
-      auto s = list_db.PopMulti(args_[1], left_, count_, &elems);
+      auto s = list_db.PopMulti(ctx, args_[1], left_, count_, &elems);
       if (!s.ok() && !s.IsNotFound()) {
         return {Status::RedisExecErr, s.ToString()};
       }
@@ -125,7 +127,7 @@ class CommandPop : public Commander {
       }
     } else {
       std::string elem;
-      auto s = list_db.Pop(args_[1], left_, &elem);
+      auto s = list_db.Pop(ctx, args_[1], left_, &elem);
       if (!s.ok() && !s.IsNotFound()) {
         return {Status::RedisExecErr, s.ToString()};
       }
@@ -197,8 +199,9 @@ class CommandLMPop : public Commander {
 
     std::vector<std::string> elems;
     std::string chosen_key;
+    engine::Context ctx(srv->storage);
     for (auto &key : keys_) {
-      auto s = list_db.PopMulti(key, left_, count_, &elems);
+      auto s = list_db.PopMulti(ctx, key, left_, count_, &elems);
       if (!s.ok() && !s.IsNotFound()) {
         return {Status::RedisExecErr, s.ToString()};
       }
@@ -262,8 +265,8 @@ class CommandBPop : public BlockingCommander {
   Status Execute(Server *srv, Connection *conn, std::string *output) override {
     srv_ = srv;
     InitConnection(conn);
-
-    auto s = TryPopFromList();
+    engine::Context ctx(srv->storage);
+    auto s = TryPopFromList(ctx);
     if (s.ok() || !s.IsNotFound()) {
       return Status::OK();  // error has already output in TryPopFromList
     }
@@ -283,14 +286,14 @@ class CommandBPop : public BlockingCommander {
     }
   }
 
-  rocksdb::Status TryPopFromList() {
+  rocksdb::Status TryPopFromList(engine::Context &ctx) {
     redis::List list_db(srv_->storage, conn_->GetNamespace());
     std::string elem;
     const std::string *last_key_ptr = nullptr;
     rocksdb::Status s;
     for (const auto &key : keys_) {
       last_key_ptr = &key;
-      s = list_db.Pop(key, left_, &elem);
+      s = list_db.Pop(ctx, key, left_, &elem);
       if (s.ok() || !s.IsNotFound()) {
         break;
       }
@@ -311,7 +314,8 @@ class CommandBPop : public BlockingCommander {
   }
 
   bool OnBlockingWrite() override {
-    auto s = TryPopFromList();
+    engine::Context ctx(srv_->storage);
+    auto s = TryPopFromList(ctx);
     return !s.IsNotFound();
   }
 
@@ -382,8 +386,8 @@ class CommandBLMPop : public BlockingCommander {
   Status Execute(Server *srv, Connection *conn, std::string *output) override {
     srv_ = srv;
     InitConnection(conn);
-
-    auto s = ExecuteUnblocked();
+    engine::Context ctx(srv->storage);
+    auto s = ExecuteUnblocked(ctx);
     if (s.ok() || !s.IsNotFound()) {
       return Status::OK();  // error has already been output
     }
@@ -391,13 +395,13 @@ class CommandBLMPop : public BlockingCommander {
     return StartBlocking(timeout_, output);
   }
 
-  rocksdb::Status ExecuteUnblocked() {
+  rocksdb::Status ExecuteUnblocked(engine::Context &ctx) {
     redis::List list_db(srv_->storage, conn_->GetNamespace());
     std::vector<std::string> elems;
     std::string chosen_key;
     rocksdb::Status s;
     for (const auto &key : keys_) {
-      s = list_db.PopMulti(key, left_, count_, &elems);
+      s = list_db.PopMulti(ctx, key, left_, count_, &elems);
       if (s.ok() && !elems.empty()) {
         chosen_key = key;
         break;
@@ -433,7 +437,8 @@ class CommandBLMPop : public BlockingCommander {
   }
 
   bool OnBlockingWrite() override {
-    auto s = ExecuteUnblocked();
+    engine::Context ctx(srv_->storage);
+    auto s = ExecuteUnblocked(ctx);
     return !s.IsNotFound();
   }
 
@@ -472,7 +477,8 @@ class CommandLRem : public Commander {
   Status Execute(Server *srv, Connection *conn, std::string *output) override {
     uint64_t ret = 0;
     redis::List list_db(srv->storage, conn->GetNamespace());
-    auto s = list_db.Rem(args_[1], count_, args_[3], &ret);
+    engine::Context ctx(srv->storage);
+    auto s = list_db.Rem(ctx, args_[1], count_, args_[3], &ret);
     if (!s.ok() && !s.IsNotFound()) {
       return {Status::RedisExecErr, s.ToString()};
     }
@@ -501,7 +507,8 @@ class CommandLInsert : public Commander {
   Status Execute(Server *srv, Connection *conn, std::string *output) override {
     int ret = 0;
     redis::List list_db(srv->storage, conn->GetNamespace());
-    auto s = list_db.Insert(args_[1], args_[3], args_[4], before_, &ret);
+    engine::Context ctx(srv->storage);
+    auto s = list_db.Insert(ctx, args_[1], args_[3], args_[4], before_, &ret);
     if (!s.ok() && !s.IsNotFound()) {
       return {Status::RedisExecErr, s.ToString()};
     }
@@ -531,7 +538,8 @@ class CommandLRange : public Commander {
   Status Execute(Server *srv, Connection *conn, std::string *output) override {
     redis::List list_db(srv->storage, conn->GetNamespace());
     std::vector<std::string> elems;
-    auto s = list_db.Range(args_[1], start_, stop_, &elems);
+    engine::Context ctx(srv->storage);
+    auto s = list_db.Range(ctx, args_[1], start_, stop_, &elems);
     if (!s.ok() && !s.IsNotFound()) {
       return {Status::RedisExecErr, s.ToString()};
     }
@@ -549,7 +557,9 @@ class CommandLLen : public Commander {
   Status Execute(Server *srv, Connection *conn, std::string *output) override {
     redis::List list_db(srv->storage, conn->GetNamespace());
     uint64_t count = 0;
-    auto s = list_db.Size(args_[1], &count);
+    engine::Context ctx(srv->storage);
+
+    auto s = list_db.Size(ctx, args_[1], &count);
     if (!s.ok() && !s.IsNotFound()) {
       return {Status::RedisExecErr, s.ToString()};
     }
@@ -574,7 +584,8 @@ class CommandLIndex : public Commander {
   Status Execute(Server *srv, Connection *conn, std::string *output) override {
     redis::List list_db(srv->storage, conn->GetNamespace());
     std::string elem;
-    auto s = list_db.Index(args_[1], index_, &elem);
+    engine::Context ctx(srv->storage);
+    auto s = list_db.Index(ctx, args_[1], index_, &elem);
     if (!s.ok() && !s.IsNotFound()) {
       return {Status::RedisExecErr, s.ToString()};
     }
@@ -605,7 +616,8 @@ class CommandLSet : public Commander {
 
   Status Execute(Server *srv, Connection *conn, std::string *output) override {
     redis::List list_db(srv->storage, conn->GetNamespace());
-    auto s = list_db.Set(args_[1], index_, args_[3]);
+    engine::Context ctx(srv->storage);
+    auto s = list_db.Set(ctx, args_[1], index_, args_[3]);
     if (!s.ok() && !s.IsNotFound()) {
       return {Status::RedisExecErr, s.ToString()};
     }
@@ -639,7 +651,8 @@ class CommandLTrim : public Commander {
 
   Status Execute(Server *srv, Connection *conn, std::string *output) override {
     redis::List list_db(srv->storage, conn->GetNamespace());
-    auto s = list_db.Trim(args_[1], start_, stop_);
+    engine::Context ctx(srv->storage);
+    auto s = list_db.Trim(ctx, args_[1], start_, stop_);
     if (!s.ok()) {
       return {Status::RedisExecErr, s.ToString()};
     }
@@ -658,7 +671,8 @@ class CommandRPopLPUSH : public Commander {
   Status Execute(Server *srv, Connection *conn, std::string *output) override {
     redis::List list_db(srv->storage, conn->GetNamespace());
     std::string elem;
-    auto s = list_db.LMove(args_[1], args_[2], /*src_left=*/false, /*dst_left=*/true, &elem);
+    engine::Context ctx(srv->storage);
+    auto s = list_db.LMove(ctx, args_[1], args_[2], /*src_left=*/false, /*dst_left=*/true, &elem);
     if (!s.ok() && !s.IsNotFound()) {
       return {Status::RedisExecErr, s.ToString()};
     }
@@ -670,7 +684,7 @@ class CommandRPopLPUSH : public Commander {
 
 class CommandLMove : public Commander {
  public:
-  Status Parse(const std::vector<std::string> &args) override {
+  Status Parse([[maybe_unused]] const std::vector<std::string> &args) override {
     auto arg_val = util::ToLower(args_[3]);
     if (arg_val != "left" && arg_val != "right") {
       return {Status::RedisParseErr, errInvalidSyntax};
@@ -689,7 +703,8 @@ class CommandLMove : public Commander {
   Status Execute(Server *srv, Connection *conn, std::string *output) override {
     redis::List list_db(srv->storage, conn->GetNamespace());
     std::string elem;
-    auto s = list_db.LMove(args_[1], args_[2], src_left_, dst_left_, &elem);
+    engine::Context ctx(srv->storage);
+    auto s = list_db.LMove(ctx, args_[1], args_[2], src_left_, dst_left_, &elem);
     if (!s.ok() && !s.IsNotFound()) {
       return {Status::RedisExecErr, s.ToString()};
     }
@@ -736,7 +751,8 @@ class CommandBLMove : public BlockingCommander {
 
     redis::List list_db(srv->storage, conn->GetNamespace());
     std::string elem;
-    auto s = list_db.LMove(args_[1], args_[2], src_left_, dst_left_, &elem);
+    engine::Context ctx(srv->storage);
+    auto s = list_db.LMove(ctx, args_[1], args_[2], src_left_, dst_left_, &elem);
     if (!s.ok() && !s.IsNotFound()) {
       return {Status::RedisExecErr, s.ToString()};
     }
@@ -755,7 +771,8 @@ class CommandBLMove : public BlockingCommander {
   bool OnBlockingWrite() override {
     redis::List list_db(srv_->storage, conn_->GetNamespace());
     std::string elem;
-    auto s = list_db.LMove(args_[1], args_[2], src_left_, dst_left_, &elem);
+    engine::Context ctx(srv_->storage);
+    auto s = list_db.LMove(ctx, args_[1], args_[2], src_left_, dst_left_, &elem);
     if (!s.ok() && !s.IsNotFound()) {
       conn_->Reply(redis::Error({Status::NotOK, s.ToString()}));
       return true;
@@ -818,7 +835,8 @@ class CommandLPos : public Commander {
   Status Execute(Server *srv, Connection *conn, std::string *output) override {
     redis::List list_db(srv->storage, conn->GetNamespace());
     std::vector<int64_t> indexes;
-    auto s = list_db.Pos(args_[1], args_[2], spec_, &indexes);
+    engine::Context ctx(srv->storage);
+    auto s = list_db.Pos(ctx, args_[1], args_[2], spec_, &indexes);
     if (!s.ok() && !s.IsNotFound()) {
       return {Status::RedisExecErr, s.ToString()};
     }
@@ -848,11 +866,11 @@ class CommandLPos : public Commander {
   PosSpec spec_;
 };
 
-REDIS_REGISTER_COMMANDS(MakeCmdAttr<CommandBLPop>("blpop", -3, "write no-script", 1, -2, 1),
+REDIS_REGISTER_COMMANDS(List, MakeCmdAttr<CommandBLPop>("blpop", -3, "write no-script", 1, -2, 1),
                         MakeCmdAttr<CommandBRPop>("brpop", -3, "write no-script", 1, -2, 1),
                         MakeCmdAttr<CommandBLMPop>("blmpop", -5, "write no-script", CommandBLMPop::keyRangeGen),
                         MakeCmdAttr<CommandLIndex>("lindex", 3, "read-only", 1, 1, 1),
-                        MakeCmdAttr<CommandLInsert>("linsert", 5, "write", 1, 1, 1),
+                        MakeCmdAttr<CommandLInsert>("linsert", 5, "write slow", 1, 1, 1),
                         MakeCmdAttr<CommandLLen>("llen", 2, "read-only", 1, 1, 1),
                         MakeCmdAttr<CommandLMove>("lmove", 5, "write", 1, 2, 1),
                         MakeCmdAttr<CommandBLMove>("blmove", 6, "write", 1, 2, 1),
@@ -861,7 +879,7 @@ REDIS_REGISTER_COMMANDS(MakeCmdAttr<CommandBLPop>("blpop", -3, "write no-script"
                         MakeCmdAttr<CommandLPush>("lpush", -3, "write", 1, 1, 1),
                         MakeCmdAttr<CommandLPushX>("lpushx", -3, "write", 1, 1, 1),
                         MakeCmdAttr<CommandLRange>("lrange", 4, "read-only", 1, 1, 1),
-                        MakeCmdAttr<CommandLRem>("lrem", 4, "write no-dbsize-check", 1, 1, 1),
+                        MakeCmdAttr<CommandLRem>("lrem", 4, "write no-dbsize-check slow", 1, 1, 1),
                         MakeCmdAttr<CommandLSet>("lset", 4, "write", 1, 1, 1),
                         MakeCmdAttr<CommandLTrim>("ltrim", 4, "write no-dbsize-check", 1, 1, 1),
                         MakeCmdAttr<CommandLMPop>("lmpop", -4, "write", CommandLMPop::keyRangeGen),
