@@ -271,7 +271,7 @@ Status SockSendFileImpl(FD out_fd, int in_fd, size_t size, Args... args) {
 // please note that, the out socket fd should be in blocking mode.
 Status SockSendFile(int out_fd, int in_fd, size_t size) { return SockSendFileImpl<SendFileImpl>(out_fd, in_fd, size); }
 
-Status SockSendFile(int out_fd, int in_fd, size_t size, ssl_st *ssl) {
+Status SockSendFile(int out_fd, int in_fd, size_t size, [[maybe_unused]] ssl_st *ssl) {
 #ifdef ENABLE_OPENSSL
   if (ssl) {
 #if OPENSSL_VERSION_NUMBER >= 0x30000000L
@@ -284,7 +284,7 @@ Status SockSendFile(int out_fd, int in_fd, size_t size, ssl_st *ssl) {
   return SockSendFile(out_fd, in_fd, size);
 }
 
-Status SockSendFile(int out_fd, int in_fd, size_t size, bufferevent *bev) {
+Status SockSendFile(int out_fd, int in_fd, size_t size, [[maybe_unused]] bufferevent *bev) {
 #ifdef ENABLE_OPENSSL
   return SockSendFile(out_fd, in_fd, size, bufferevent_openssl_get_ssl(bev));
 #else
@@ -454,7 +454,7 @@ Status Write(int fd, const std::string &data) { return WriteImpl<write>(fd, data
 
 Status Pwrite(int fd, const std::string &data, off_t offset) { return WriteImpl<pwrite>(fd, data, offset); }
 
-Status SockSend(int fd, const std::string &data, ssl_st *ssl) {
+Status SockSend(int fd, const std::string &data, [[maybe_unused]] ssl_st *ssl) {
 #ifdef ENABLE_OPENSSL
   if (ssl) {
     return WriteImpl<SSL_write>(ssl, data);
@@ -463,7 +463,7 @@ Status SockSend(int fd, const std::string &data, ssl_st *ssl) {
   return SockSend(fd, data);
 }
 
-Status SockSend(int fd, const std::string &data, bufferevent *bev) {
+Status SockSend(int fd, const std::string &data, [[maybe_unused]] bufferevent *bev) {
 #ifdef ENABLE_OPENSSL
   return SockSend(fd, data, bufferevent_openssl_get_ssl(bev));
 #else
@@ -471,7 +471,8 @@ Status SockSend(int fd, const std::string &data, bufferevent *bev) {
 #endif
 }
 
-StatusOr<int> SockConnect(const std::string &host, uint32_t port, ssl_st *ssl, int conn_timeout, int timeout) {
+StatusOr<int> SockConnect(const std::string &host, uint32_t port, [[maybe_unused]] ssl_st *ssl, int conn_timeout,
+                          int timeout) {
 #ifdef ENABLE_OPENSSL
   if (ssl) {
     auto fd = GET_OR_RET(SockConnect(host, port, conn_timeout, timeout));
@@ -491,7 +492,7 @@ StatusOr<int> SockConnect(const std::string &host, uint32_t port, ssl_st *ssl, i
   return SockConnect(host, port, conn_timeout, timeout);
 }
 
-StatusOr<int> EvbufferRead(evbuffer *buf, evutil_socket_t fd, int howmuch, ssl_st *ssl) {
+StatusOr<int> EvbufferRead(evbuffer *buf, evutil_socket_t fd, int howmuch, [[maybe_unused]] ssl_st *ssl) {
 #ifdef ENABLE_OPENSSL
   if (ssl) {
     constexpr int BUFFER_SIZE = 4096;
