@@ -90,34 +90,31 @@ rocksdb::Status Database::ParseMetadata(RedisTypes types, Slice *bytes, Metadata
   return s;
 }
 
-rocksdb::Status Database::GetMetadata(GetOptions options, RedisTypes types, const Slice &ns_key, Metadata *metadata) {
+rocksdb::Status Database::GetMetadata(engine::Context &ctx, RedisTypes types, const Slice &ns_key, Metadata *metadata) {
   std::string raw_value;
   Slice rest;
-  return GetMetadata(options, types, ns_key, &raw_value, metadata, &rest);
+  return GetMetadata(ctx, types, ns_key, &raw_value, metadata, &rest);
 }
 
-rocksdb::Status Database::GetMetadata(GetOptions options, RedisTypes types, const Slice &ns_key, std::string *raw_value,
-                                      Metadata *metadata, Slice *rest) {
-  auto s = GetRawMetadata(options, ns_key, raw_value);
+rocksdb::Status Database::GetMetadata(engine::Context &ctx, RedisTypes types, const Slice &ns_key,
+                                      std::string *raw_value, Metadata *metadata, Slice *rest) {
+  auto s = GetRawMetadata(ctx, ns_key, raw_value);
   *rest = *raw_value;
   if (!s.ok()) return s;
   return ParseMetadata(types, rest, metadata);
 }
 
-rocksdb::Status Database::GetRawMetadata(GetOptions options, const Slice &ns_key, std::string *bytes) {
-  rocksdb::ReadOptions opts;
-  // If options.snapshot == nullptr, we can avoid allocating a snapshot here.
-  opts.snapshot = options.snapshot;
-  return storage_->Get(opts, metadata_cf_handle_, ns_key, bytes);
+rocksdb::Status Database::GetRawMetadata(engine::Context &ctx, const Slice &ns_key, std::string *bytes) {
+  return storage_->Get(ctx, ctx.GetReadOptions(), metadata_cf_handle_, ns_key, bytes);
 }
 
-rocksdb::Status Database::Expire(const Slice &user_key, uint64_t timestamp) {
+rocksdb::Status Database::Expire(engine::Context &ctx, const Slice &user_key, uint64_t timestamp) {
   std::string ns_key = AppendNamespacePrefix(user_key);
 
   std::string value;
   Metadata metadata(kRedisNone, false);
   LockGuard guard(storage_->GetLockManager(), ns_key);
-  rocksdb::Status s = storage_->Get(rocksdb::ReadOptions(), metadata_cf_handle_, ns_key, &value);
+  rocksdb::Status s = storage_->Get(ctx, ctx.GetReadOptions(), metadata_cf_handle_, ns_key, &value);
   if (!s.ok()) return s;
 
   s = metadata.Decode(value);
@@ -138,18 +135,24 @@ rocksdb::Status Database::Expire(const Slice &user_key, uint64_t timestamp) {
   }
   auto batch = storage_->GetWriteBatchBase();
   WriteBatchLogData log_data(kRedisNone, {std::to_string(kRedisCmdExpire)});
-  batch->PutLogData(log_data.Encode());
-  batch->Put(metadata_cf_handle_, ns_key, value);
-  s = storage_->Write(storage_->DefaultWriteOptions(), batch->GetWriteBatch());
+  s = batch->PutLogData(log_data.Encode());
+  if (!s.ok()) {
+    return s;
+  }
+  s = batch->Put(metadata_cf_handle_, ns_key, value);
+  if (!s.ok()) {
+    return s;
+  }
+  s = storage_->Write(ctx, storage_->DefaultWriteOptions(), batch->GetWriteBatch());
   return s;
 }
 
-rocksdb::Status Database::Del(const Slice &user_key) {
+rocksdb::Status Database::Del(engine::Context &ctx, const Slice &user_key) {
   std::string ns_key = AppendNamespacePrefix(user_key);
 
   std::string value;
   LockGuard guard(storage_->GetLockManager(), ns_key);
-  rocksdb::Status s = storage_->Get(rocksdb::ReadOptions(), metadata_cf_handle_, ns_key, &value);
+  rocksdb::Status s = storage_->Get(ctx, ctx.GetReadOptions(), metadata_cf_handle_, ns_key, &value);
   if (!s.ok()) return s;
   Metadata metadata(kRedisNone, false);
   s = metadata.Decode(value);
@@ -157,10 +160,10 @@ rocksdb::Status Database::Del(const Slice &user_key) {
   if (metadata.Expired()) {
     return rocksdb::Status::NotFound(kErrMsgKeyExpired);
   }
-  return storage_->Delete(storage_->DefaultWriteOptions(), metadata_cf_handle_, ns_key);
+  return storage_->Delete(ctx, storage_->DefaultWriteOptions(), metadata_cf_handle_, ns_key);
 }
 
-rocksdb::Status Database::MDel(const std::vector<Slice> &keys, uint64_t *deleted_cnt) {
+rocksdb::Status Database::MDel(engine::Context &ctx, const std::vector<Slice> &keys, uint64_t *deleted_cnt) {
   *deleted_cnt = 0;
 
   std::vector<std::string> lock_keys;
@@ -173,7 +176,10 @@ rocksdb::Status Database::MDel(const std::vector<Slice> &keys, uint64_t *deleted
 
   auto batch = storage_->GetWriteBatchBase();
   WriteBatchLogData log_data(kRedisNone);
-  batch->PutLogData(log_data.Encode());
+  auto s = batch->PutLogData(log_data.Encode());
+  if (!s.ok()) {
+    return s;
+  }
 
   std::vector<Slice> slice_keys;
   slice_keys.reserve(lock_keys.size());
@@ -181,13 +187,10 @@ rocksdb::Status Database::MDel(const std::vector<Slice> &keys, uint64_t *deleted
     slice_keys.emplace_back(ns_key);
   }
 
-  LatestSnapShot ss(storage_);
-  rocksdb::ReadOptions read_options = storage_->DefaultMultiGetOptions();
-  read_options.snapshot = ss.GetSnapShot();
   std::vector<rocksdb::Status> statuses(slice_keys.size());
   std::vector<rocksdb::PinnableSlice> pin_values(slice_keys.size());
-  storage_->MultiGet(read_options, metadata_cf_handle_, slice_keys.size(), slice_keys.data(), pin_values.data(),
-                     statuses.data());
+  storage_->MultiGet(ctx, ctx.GetReadOptions(), metadata_cf_handle_, slice_keys.size(), slice_keys.data(),
+                     pin_values.data(), statuses.data());
 
   for (size_t i = 0; i < slice_keys.size(); i++) {
     if (!statuses[i].ok() && !statuses[i].IsNotFound()) return statuses[i];
@@ -200,33 +203,31 @@ rocksdb::Status Database::MDel(const std::vector<Slice> &keys, uint64_t *deleted
     if (!s.ok()) continue;
     if (metadata.Expired()) continue;
 
-    batch->Delete(metadata_cf_handle_, lock_keys[i]);
+    s = batch->Delete(metadata_cf_handle_, lock_keys[i]);
+    if (!s.ok()) return s;
     *deleted_cnt += 1;
   }
 
   if (*deleted_cnt == 0) return rocksdb::Status::OK();
 
-  return storage_->Write(storage_->DefaultWriteOptions(), batch->GetWriteBatch());
+  return storage_->Write(ctx, storage_->DefaultWriteOptions(), batch->GetWriteBatch());
 }
 
-rocksdb::Status Database::Exists(const std::vector<Slice> &keys, int *ret) {
+rocksdb::Status Database::Exists(engine::Context &ctx, const std::vector<Slice> &keys, int *ret) {
   std::vector<std::string> ns_keys;
   ns_keys.reserve(keys.size());
   for (const auto &key : keys) {
     ns_keys.emplace_back(AppendNamespacePrefix(key));
   }
-  return existsInternal(ns_keys, ret);
+  return existsInternal(ctx, ns_keys, ret);
 }
 
-rocksdb::Status Database::TTL(const Slice &user_key, int64_t *ttl) {
+rocksdb::Status Database::TTL(engine::Context &ctx, const Slice &user_key, int64_t *ttl) {
   std::string ns_key = AppendNamespacePrefix(user_key);
 
   *ttl = -2;  // ttl is -2 when the key does not exist or expired
-  LatestSnapShot ss(storage_);
-  rocksdb::ReadOptions read_options;
-  read_options.snapshot = ss.GetSnapShot();
   std::string value;
-  rocksdb::Status s = storage_->Get(read_options, metadata_cf_handle_, ns_key, &value);
+  rocksdb::Status s = storage_->Get(ctx, ctx.GetReadOptions(), metadata_cf_handle_, ns_key, &value);
   if (!s.ok()) return s.IsNotFound() ? rocksdb::Status::OK() : s;
 
   Metadata metadata(kRedisNone, false);
@@ -237,21 +238,22 @@ rocksdb::Status Database::TTL(const Slice &user_key, int64_t *ttl) {
   return rocksdb::Status::OK();
 }
 
-rocksdb::Status Database::GetExpireTime(const Slice &user_key, uint64_t *timestamp) {
+rocksdb::Status Database::GetExpireTime(engine::Context &ctx, const Slice &user_key, uint64_t *timestamp) {
   std::string ns_key = AppendNamespacePrefix(user_key);
   Metadata metadata(kRedisNone, false);
-  auto s = GetMetadata(GetOptions{}, RedisTypes::All(), ns_key, &metadata);
+  auto s = GetMetadata(ctx, RedisTypes::All(), ns_key, &metadata);
   if (!s.ok()) return s;
   *timestamp = metadata.expire;
 
   return rocksdb::Status::OK();
 }
 
-rocksdb::Status Database::GetKeyNumStats(const std::string &prefix, KeyNumStats *stats) {
-  return Keys(prefix, nullptr, stats);
+rocksdb::Status Database::GetKeyNumStats(engine::Context &ctx, const std::string &prefix, KeyNumStats *stats) {
+  return Keys(ctx, prefix, nullptr, stats);
 }
 
-rocksdb::Status Database::Keys(const std::string &prefix, std::vector<std::string> *keys, KeyNumStats *stats) {
+rocksdb::Status Database::Keys(engine::Context &ctx, const std::string &prefix, std::vector<std::string> *keys,
+                               KeyNumStats *stats) {
   uint16_t slot_id = 0;
   std::string ns_prefix;
   if (namespace_ != kDefaultNamespace || keys != nullptr) {
@@ -267,10 +269,7 @@ rocksdb::Status Database::Keys(const std::string &prefix, std::vector<std::strin
   }
 
   uint64_t ttl_sum = 0;
-  LatestSnapShot ss(storage_);
-  rocksdb::ReadOptions read_options = storage_->DefaultScanOptions();
-  read_options.snapshot = ss.GetSnapShot();
-  auto iter = util::UniqueIterator(storage_, read_options, metadata_cf_handle_);
+  auto iter = util::UniqueIterator(ctx, ctx.GetReadOptions(), metadata_cf_handle_);
 
   while (true) {
     ns_prefix.empty() ? iter->SeekToFirst() : iter->Seek(ns_prefix);
@@ -299,6 +298,10 @@ rocksdb::Status Database::Keys(const std::string &prefix, std::vector<std::strin
       }
     }
 
+    if (auto s = iter->status(); !s.ok()) {
+      return s;
+    }
+
     if (!storage_->IsSlotIdEncoded()) break;
     if (prefix.empty()) break;
     if (++slot_id >= HASH_SLOTS_SIZE) break;
@@ -315,18 +318,16 @@ rocksdb::Status Database::Keys(const std::string &prefix, std::vector<std::strin
   return rocksdb::Status::OK();
 }
 
-rocksdb::Status Database::Scan(const std::string &cursor, uint64_t limit, const std::string &prefix,
-                               std::vector<std::string> *keys, std::string *end_cursor, RedisType type) {
+rocksdb::Status Database::Scan(engine::Context &ctx, const std::string &cursor, uint64_t limit,
+                               const std::string &prefix, std::vector<std::string> *keys, std::string *end_cursor,
+                               RedisType type) {
   end_cursor->clear();
   uint64_t cnt = 0;
   uint16_t slot_start = 0;
   std::string ns_prefix;
   std::string user_key;
 
-  LatestSnapShot ss(storage_);
-  rocksdb::ReadOptions read_options = storage_->DefaultScanOptions();
-  read_options.snapshot = ss.GetSnapShot();
-  auto iter = util::UniqueIterator(storage_, read_options, metadata_cf_handle_);
+  auto iter = util::UniqueIterator(ctx, ctx.GetReadOptions(), metadata_cf_handle_);
 
   std::string ns_cursor = AppendNamespacePrefix(cursor);
   if (storage_->IsSlotIdEncoded()) {
@@ -368,6 +369,11 @@ rocksdb::Status Database::Scan(const std::string &cursor, uint64_t limit, const
       keys->emplace_back(user_key);
       cnt++;
     }
+
+    if (auto s = iter->status(); !s.ok()) {
+      return s;
+    }
+
     if (!storage_->IsSlotIdEncoded() || prefix.empty()) {
       if (!keys->empty() && cnt >= limit) {
         end_cursor->append(user_key);
@@ -409,18 +415,18 @@ rocksdb::Status Database::Scan(const std::string &cursor, uint64_t limit, const
   return rocksdb::Status::OK();
 }
 
-rocksdb::Status Database::RandomKey(const std::string &cursor, std::string *key) {
+rocksdb::Status Database::RandomKey(engine::Context &ctx, const std::string &cursor, std::string *key) {
   key->clear();
 
   std::string end_cursor;
   std::vector<std::string> keys;
-  auto s = Scan(cursor, RANDOM_KEY_SCAN_LIMIT, "", &keys, &end_cursor);
+  auto s = Scan(ctx, cursor, RANDOM_KEY_SCAN_LIMIT, "", &keys, &end_cursor);
   if (!s.ok()) {
     return s;
   }
   if (keys.empty() && !cursor.empty()) {
     // if reach the end, restart from beginning
-    s = Scan("", RANDOM_KEY_SCAN_LIMIT, "", &keys, &end_cursor);
+    s = Scan(ctx, "", RANDOM_KEY_SCAN_LIMIT, "", &keys, &end_cursor);
     if (!s.ok()) {
       return s;
     }
@@ -432,18 +438,15 @@ rocksdb::Status Database::RandomKey(const std::string &cursor, std::string *key)
   return rocksdb::Status::OK();
 }
 
-rocksdb::Status Database::FlushDB() {
+rocksdb::Status Database::FlushDB(engine::Context &ctx) {
   auto begin_key = ComposeNamespaceKey(namespace_, "", false);
   auto end_key = util::StringNext(begin_key);
 
-  return storage_->DeleteRange(begin_key, end_key);
+  return storage_->DeleteRange(ctx, begin_key, end_key);
 }
 
-rocksdb::Status Database::FlushAll() {
-  LatestSnapShot ss(storage_);
-  rocksdb::ReadOptions read_options = storage_->DefaultScanOptions();
-  read_options.snapshot = ss.GetSnapShot();
-  auto iter = util::UniqueIterator(storage_, read_options, metadata_cf_handle_);
+rocksdb::Status Database::FlushAll(engine::Context &ctx) {
+  auto iter = util::UniqueIterator(ctx, ctx.GetReadOptions(), metadata_cf_handle_);
   iter->SeekToFirst();
   if (!iter->Valid()) {
     return rocksdb::Status::OK();
@@ -454,19 +457,16 @@ rocksdb::Status Database::FlushAll() {
     return rocksdb::Status::OK();
   }
   auto last_key = util::StringNext(iter->key().ToString());
-  return storage_->DeleteRange(first_key, last_key);
+  return storage_->DeleteRange(ctx, first_key, last_key);
 }
 
-rocksdb::Status Database::Dump(const Slice &user_key, std::vector<std::string> *infos) {
+rocksdb::Status Database::Dump(engine::Context &ctx, const Slice &user_key, std::vector<std::string> *infos) {
   infos->clear();
 
   std::string ns_key = AppendNamespacePrefix(user_key);
 
-  LatestSnapShot ss(storage_);
-  rocksdb::ReadOptions read_options;
-  read_options.snapshot = ss.GetSnapShot();
   std::string value;
-  rocksdb::Status s = storage_->Get(read_options, metadata_cf_handle_, ns_key, &value);
+  rocksdb::Status s = storage_->Get(ctx, ctx.GetReadOptions(), metadata_cf_handle_, ns_key, &value);
   if (!s.ok()) return s.IsNotFound() ? rocksdb::Status::OK() : s;
 
   Metadata metadata(kRedisNone, false);
@@ -500,7 +500,7 @@ rocksdb::Status Database::Dump(const Slice &user_key, std::vector<std::string> *
 
   if (metadata.Type() == kRedisList) {
     ListMetadata list_metadata(false);
-    s = GetMetadata(GetOptions{}, {kRedisList}, ns_key, &list_metadata);
+    s = GetMetadata(ctx, {kRedisList}, ns_key, &list_metadata);
     if (!s.ok()) return s.IsNotFound() ? rocksdb::Status::OK() : s;
     infos->emplace_back("head");
     infos->emplace_back(std::to_string(list_metadata.head));
@@ -511,33 +511,30 @@ rocksdb::Status Database::Dump(const Slice &user_key, std::vector<std::string> *
   return rocksdb::Status::OK();
 }
 
-rocksdb::Status Database::Type(const Slice &key, RedisType *type) {
+rocksdb::Status Database::Type(engine::Context &ctx, const Slice &key, RedisType *type) {
   std::string ns_key = AppendNamespacePrefix(key);
-  return typeInternal(ns_key, type);
+  return typeInternal(ctx, ns_key, type);
 }
 
 std::string Database::AppendNamespacePrefix(const Slice &user_key) {
   return ComposeNamespaceKey(namespace_, user_key, storage_->IsSlotIdEncoded());
 }
 
-rocksdb::Status Database::ClearKeysOfSlot(const rocksdb::Slice &ns, int slot) {
+rocksdb::Status Database::ClearKeysOfSlotRange(engine::Context &ctx, const rocksdb::Slice &ns,
+                                               const SlotRange &slot_range) {
   if (!storage_->IsSlotIdEncoded()) {
     return rocksdb::Status::Aborted("It is not in cluster mode");
   }
 
-  std::string prefix = ComposeSlotKeyPrefix(ns, slot);
-  std::string prefix_end = ComposeSlotKeyPrefix(ns, slot + 1);
-  auto s = storage_->DeleteRange(prefix, prefix_end);
-  if (!s.ok()) {
-    return s;
-  }
-  return rocksdb::Status::OK();
+  std::string prefix = ComposeSlotKeyPrefix(ns, slot_range.start);
+  std::string prefix_end = ComposeSlotKeyPrefix(ns, slot_range.end + 1);
+  return storage_->DeleteRange(ctx, prefix, prefix_end);
 }
 
-rocksdb::Status Database::KeyExist(const std::string &key) {
+rocksdb::Status Database::KeyExist(engine::Context &ctx, const std::string &key) {
   int cnt = 0;
   std::vector<rocksdb::Slice> keys{key};
-  auto s = Exists(keys, &cnt);
+  auto s = Exists(ctx, keys, &cnt);
   if (!s.ok()) {
     return s;
   }
@@ -547,18 +544,16 @@ rocksdb::Status Database::KeyExist(const std::string &key) {
   return rocksdb::Status::OK();
 }
 
-rocksdb::Status SubKeyScanner::Scan(RedisType type, const Slice &user_key, const std::string &cursor, uint64_t limit,
-                                    const std::string &subkey_prefix, std::vector<std::string> *keys,
-                                    std::vector<std::string> *values) {
+rocksdb::Status SubKeyScanner::Scan(engine::Context &ctx, RedisType type, const Slice &user_key,
+                                    const std::string &cursor, uint64_t limit, const std::string &subkey_prefix,
+                                    std::vector<std::string> *keys, std::vector<std::string> *values) {
   uint64_t cnt = 0;
   std::string ns_key = AppendNamespacePrefix(user_key);
   Metadata metadata(type, false);
-  LatestSnapShot ss(storage_);
-  rocksdb::Status s = GetMetadata(GetOptions{ss.GetSnapShot()}, {type}, ns_key, &metadata);
+  rocksdb::Status s = GetMetadata(ctx, {type}, ns_key, &metadata);
   if (!s.ok()) return s;
 
-  rocksdb::ReadOptions read_options = storage_->DefaultScanOptions();
-  auto iter = util::UniqueIterator(storage_, read_options);
+  auto iter = util::UniqueIterator(ctx, ctx.DefaultScanOptions());
   std::string match_prefix_key =
       InternalKey(ns_key, subkey_prefix, metadata.version, storage_->IsSlotIdEncoded()).Encode();
 
@@ -587,7 +582,7 @@ rocksdb::Status SubKeyScanner::Scan(RedisType type, const Slice &user_key, const
       break;
     }
   }
-  return rocksdb::Status::OK();
+  return iter->status();
 }
 
 RedisType WriteBatchLogData::GetRedisType() const { return type_; }
@@ -615,16 +610,12 @@ Status WriteBatchLogData::Decode(const rocksdb::Slice &blob) {
   return Status::OK();
 }
 
-rocksdb::Status Database::existsInternal(const std::vector<std::string> &keys, int *ret) {
+rocksdb::Status Database::existsInternal(engine::Context &ctx, const std::vector<std::string> &keys, int *ret) {
   *ret = 0;
-  LatestSnapShot ss(storage_);
-  rocksdb::ReadOptions read_options;
-  read_options.snapshot = ss.GetSnapShot();
-
   rocksdb::Status s;
   std::string value;
   for (const auto &key : keys) {
-    s = storage_->Get(read_options, metadata_cf_handle_, key, &value);
+    s = storage_->Get(ctx, ctx.GetReadOptions(), metadata_cf_handle_, key, &value);
     if (!s.ok() && !s.IsNotFound()) return s;
     if (s.ok()) {
       Metadata metadata(kRedisNone, false);
@@ -636,13 +627,10 @@ rocksdb::Status Database::existsInternal(const std::vector<std::string> &keys, i
   return rocksdb::Status::OK();
 }
 
-rocksdb::Status Database::typeInternal(const Slice &key, RedisType *type) {
+rocksdb::Status Database::typeInternal(engine::Context &ctx, const Slice &key, RedisType *type) {
   *type = kRedisNone;
-  LatestSnapShot ss(storage_);
-  rocksdb::ReadOptions read_options;
-  read_options.snapshot = ss.GetSnapShot();
   std::string value;
-  rocksdb::Status s = storage_->Get(read_options, metadata_cf_handle_, key, &value);
+  rocksdb::Status s = storage_->Get(ctx, ctx.GetReadOptions(), metadata_cf_handle_, key, &value);
   if (!s.ok()) return s.IsNotFound() ? rocksdb::Status::OK() : s;
 
   Metadata metadata(kRedisNone, false);
@@ -656,13 +644,13 @@ rocksdb::Status Database::typeInternal(const Slice &key, RedisType *type) {
   return rocksdb::Status::OK();
 }
 
-rocksdb::Status Database::Copy(const std::string &key, const std::string &new_key, bool nx, bool delete_old,
-                               CopyResult *res) {
+rocksdb::Status Database::Copy(engine::Context &ctx, const std::string &key, const std::string &new_key, bool nx,
+                               bool delete_old, CopyResult *res) {
   std::vector<std::string> lock_keys = {key, new_key};
   MultiLockGuard guard(storage_->GetLockManager(), lock_keys);
 
   RedisType type = kRedisNone;
-  auto s = typeInternal(key, &type);
+  auto s = typeInternal(ctx, key, &type);
   if (!s.ok()) return s;
   if (type == kRedisNone) {
     *res = CopyResult::KEY_NOT_EXIST;
@@ -671,7 +659,7 @@ rocksdb::Status Database::Copy(const std::string &key, const std::string &new_ke
 
   if (nx) {
     int exist = 0;
-    if (s = existsInternal({new_key}, &exist), !s.ok()) return s;
+    if (s = existsInternal(ctx, {new_key}, &exist), !s.ok()) return s;
     if (exist > 0) {
       *res = CopyResult::KEY_ALREADY_EXIST;
       return rocksdb::Status::OK();
@@ -684,16 +672,25 @@ rocksdb::Status Database::Copy(const std::string &key, const std::string &new_ke
 
   auto batch = storage_->GetWriteBatchBase();
   WriteBatchLogData log_data(type);
-  batch->PutLogData(log_data.Encode());
+  s = batch->PutLogData(log_data.Encode());
+  if (!s.ok()) {
+    return s;
+  }
 
-  engine::DBIterator iter(storage_, rocksdb::ReadOptions());
+  engine::DBIterator iter(ctx, ctx.GetReadOptions());
   iter.Seek(key);
 
   if (delete_old) {
-    batch->Delete(metadata_cf_handle_, key);
+    s = batch->Delete(metadata_cf_handle_, key);
+    if (!s.ok()) {
+      return s;
+    }
   }
   // copy metadata
-  batch->Put(metadata_cf_handle_, new_key, iter.Value());
+  s = batch->Put(metadata_cf_handle_, new_key, iter.Value());
+  if (!s.ok()) {
+    return s;
+  }
 
   auto subkey_iter = iter.GetSubKeyIterator();
 
@@ -705,7 +702,10 @@ rocksdb::Status Database::Copy(const std::string &key, const std::string &new_ke
       std::string to_ikey =
           InternalKey(new_key, from_ikey.GetSubKey(), from_ikey.GetVersion(), storage_->IsSlotIdEncoded()).Encode();
       // copy sub key
-      batch->Put(subkey_iter->ColumnFamilyHandle(), to_ikey, subkey_iter->Value());
+      auto s = batch->Put(subkey_iter->ColumnFamilyHandle(), to_ikey, subkey_iter->Value());
+      if (!s.ok()) {
+        return s;
+      }
 
       // The ZSET type stores an extra score and member field inside `zset_score` column family
       // while compared to other composed data structures. The purpose is to allow to seek by score.
@@ -715,15 +715,19 @@ rocksdb::Status Database::Copy(const std::string &key, const std::string &new_ke
         // copy score key
         std::string score_key =
             InternalKey(new_key, score_bytes, from_ikey.GetVersion(), storage_->IsSlotIdEncoded()).Encode();
-        batch->Put(zset_score_cf, score_key, Slice());
+        auto s = batch->Put(zset_score_cf, score_key, Slice());
+        if (!s.ok()) {
+          return s;
+        }
       }
     }
   }
 
-  return storage_->Write(storage_->DefaultWriteOptions(), batch->GetWriteBatch());
+  return storage_->Write(ctx, storage_->DefaultWriteOptions(), batch->GetWriteBatch());
 }
 
-std::optional<std::string> Database::lookupKeyByPattern(const std::string &pattern, const std::string &subst) {
+std::optional<std::string> Database::lookupKeyByPattern(engine::Context &ctx, const std::string &pattern,
+                                                        const std::string &subst) {
   if (pattern == "#") {
     return subst;
   }
@@ -747,31 +751,31 @@ std::optional<std::string> Database::lookupKeyByPattern(const std::string &patte
   RedisType type = RedisType::kRedisNone;
   if (!field.empty()) {
     auto hash_db = redis::Hash(storage_, namespace_);
-    if (auto s = hash_db.Type(key, &type); !s.ok() || type != RedisType::kRedisHash) {
+    if (auto s = hash_db.Type(ctx, key, &type); !s.ok() || type != RedisType::kRedisHash) {
       return std::nullopt;
     }
 
-    if (auto s = hash_db.Get(key, field, &value); !s.ok()) {
+    if (auto s = hash_db.Get(ctx, key, field, &value); !s.ok()) {
       return std::nullopt;
     }
   } else {
     auto string_db = redis::String(storage_, namespace_);
-    if (auto s = string_db.Type(key, &type); !s.ok() || type != RedisType::kRedisString) {
+    if (auto s = string_db.Type(ctx, key, &type); !s.ok() || type != RedisType::kRedisString) {
       return std::nullopt;
     }
-    if (auto s = string_db.Get(key, &value); !s.ok()) {
+    if (auto s = string_db.Get(ctx, key, &value); !s.ok()) {
       return std::nullopt;
     }
   }
   return value;
 }
 
-rocksdb::Status Database::Sort(RedisType type, const std::string &key, const SortArgument &args,
+rocksdb::Status Database::Sort(engine::Context &ctx, RedisType type, const std::string &key, const SortArgument &args,
                                std::vector<std::optional<std::string>> *elems, SortResult *res) {
   // Obtain the length of the object to sort.
   const std::string ns_key = AppendNamespacePrefix(key);
   Metadata metadata(type, false);
-  auto s = GetMetadata(GetOptions{}, {type}, ns_key, &metadata);
+  auto s = GetMetadata(ctx, {type}, ns_key, &metadata);
   if (!s.ok()) return s;
 
   if (metadata.size > SORT_LENGTH_LIMIT) {
@@ -793,20 +797,20 @@ rocksdb::Status Database::Sort(RedisType type, const std::string &key, const Sor
 
       if (args.dontsort) {
         if (args.desc) {
-          s = list_db.Range(key, -count - offset, -1 - offset, &str_vec);
+          s = list_db.Range(ctx, key, -count - offset, -1 - offset, &str_vec);
           if (!s.ok()) return s;
           std::reverse(str_vec.begin(), str_vec.end());
         } else {
-          s = list_db.Range(key, offset, offset + count - 1, &str_vec);
+          s = list_db.Range(ctx, key, offset, offset + count - 1, &str_vec);
           if (!s.ok()) return s;
         }
       } else {
-        s = list_db.Range(key, 0, -1, &str_vec);
+        s = list_db.Range(ctx, key, 0, -1, &str_vec);
         if (!s.ok()) return s;
       }
     } else if (type == RedisType::kRedisSet) {
       auto set_db = redis::Set(storage_, namespace_);
-      s = set_db.Members(key, &str_vec);
+      s = set_db.Members(ctx, key, &str_vec);
       if (!s.ok()) return s;
 
       if (args.dontsort) {
@@ -822,14 +826,14 @@ rocksdb::Status Database::Sort(RedisType type, const std::string &key, const Sor
         spec.start = offset;
         spec.stop = offset + count - 1;
         spec.reversed = args.desc;
-        s = zset_db.RangeByRank(key, spec, &member_scores, nullptr);
+        s = zset_db.RangeByRank(ctx, key, spec, &member_scores, nullptr);
         if (!s.ok()) return s;
 
         for (auto &member_score : member_scores) {
           str_vec.emplace_back(std::move(member_score.member));
         }
       } else {
-        s = zset_db.GetAllMemberScores(key, &member_scores);
+        s = zset_db.GetAllMemberScores(ctx, key, &member_scores);
         if (!s.ok()) return s;
 
         for (auto &member_score : member_scores) {
@@ -852,7 +856,7 @@ rocksdb::Status Database::Sort(RedisType type, const std::string &key, const Sor
     for (size_t i = 0; i < sort_vec.size(); ++i) {
       std::string byval;
       if (!args.sortby.empty()) {
-        auto lookup = lookupKeyByPattern(args.sortby, str_vec[i]);
+        auto lookup = lookupKeyByPattern(ctx, args.sortby, str_vec[i]);
         if (!lookup.has_value()) continue;
         byval = std::move(lookup.value());
       } else {
@@ -888,7 +892,7 @@ rocksdb::Status Database::Sort(RedisType type, const std::string &key, const Sor
       elems->emplace_back(elem.obj);
     }
     for (const std::string &pattern : args.getpatterns) {
-      std::optional<std::string> val = lookupKeyByPattern(pattern, elem.obj);
+      std::optional<std::string> val = lookupKeyByPattern(ctx, pattern, elem.obj);
       if (val.has_value()) {
         elems->emplace_back(val.value());
       } else {
@@ -904,10 +908,12 @@ rocksdb::Status Database::Sort(RedisType type, const std::string &key, const Sor
       store_elems.emplace_back(e.value_or(""));
     }
     redis::List list_db(storage_, namespace_);
-    s = list_db.Trim(args.storekey, -1, 0);
+    s = list_db.Trim(ctx, args.storekey, -1, 0);
+
     if (!s.ok()) return s;
     uint64_t new_size = 0;
-    s = list_db.Push(args.storekey, std::vector<Slice>(store_elems.cbegin(), store_elems.cend()), false, &new_size);
+    s = list_db.Push(ctx, args.storekey, std::vector<Slice>(store_elems.cbegin(), store_elems.cend()), false,
+                     &new_size);
     if (!s.ok()) return s;
   }
 
