@@ -57,11 +57,11 @@ enum {
 
 namespace lua {
 
-lua_State *CreateState(Server *srv, bool read_only) {
+lua_State *CreateState(Server *srv) {
   lua_State *lua = lua_open();
   LoadLibraries(lua);
   RemoveUnsupportedFunctions(lua);
-  LoadFuncs(lua, read_only);
+  LoadFuncs(lua);
 
   lua_pushlightuserdata(lua, srv);
   lua_setglobal(lua, REDIS_LUA_SERVER_PTR);
@@ -75,7 +75,7 @@ void DestroyState(lua_State *lua) {
   lua_close(lua);
 }
 
-void LoadFuncs(lua_State *lua, bool read_only) {
+void LoadFuncs(lua_State *lua) {
   lua_newtable(lua);
 
   /* redis.call */
@@ -127,11 +127,6 @@ void LoadFuncs(lua_State *lua, bool read_only) {
   lua_pushcfunction(lua, RedisStatusReplyCommand);
   lua_settable(lua, -3);
 
-  /* redis.read_only */
-  lua_pushstring(lua, "read_only");
-  lua_pushboolean(lua, read_only);
-  lua_settable(lua, -3);
-
   /* redis.register_function */
   lua_pushstring(lua, "register_function");
   lua_pushcfunction(lua, RedisRegisterFunction);
@@ -226,8 +221,8 @@ int RedisLogCommand(lua_State *lua) {
 int RedisRegisterFunction(lua_State *lua) {
   int argc = lua_gettop(lua);
 
-  if (argc != 2) {
-    lua_pushstring(lua, "redis.register_function() requires two arguments.");
+  if (argc < 2 || argc > 3) {
+    lua_pushstring(lua, "wrong number of arguments to redis.register_function().");
     return lua_error(lua);
   }
 
@@ -243,6 +238,15 @@ int RedisRegisterFunction(lua_State *lua) {
 
   // set this function to global
   std::string name = lua_tostring(lua, 1);
+  if (argc == 3) {
+    auto flags = ExtractFlagsFromRegisterFunction(lua);
+    if (!flags) {
+      lua_pushstring(lua, flags.Msg().c_str());
+      return lua_error(lua);
+    }
+    lua_pushinteger(lua, static_cast<lua_Integer>(flags.GetValue()));
+    lua_setglobal(lua, (REDIS_LUA_REGISTER_FUNC_FLAGS_PREFIX + name).c_str());
+  }
   lua_setglobal(lua, (REDIS_LUA_REGISTER_FUNC_PREFIX + name).c_str());
 
   // set this function name to REDIS_FUNCTION_LIBRARIES[libname]
@@ -274,12 +278,11 @@ int RedisRegisterFunction(lua_State *lua) {
     lua_pushstring(lua, "redis.register_function() failed to store informantion.");
     return lua_error(lua);
   }
-
   return 0;
 }
 
 Status FunctionLoad(redis::Connection *conn, const std::string &script, bool need_to_store, bool replace,
-                    std::string *lib_name, bool read_only) {
+                    [[maybe_unused]] std::string *lib_name, bool read_only) {
   std::string first_line, lua_code;
   if (auto pos = script.find('\n'); pos != std::string::npos) {
     first_line = script.substr(0, pos);
@@ -288,31 +291,8 @@ Status FunctionLoad(redis::Connection *conn, const std::string &script, bool nee
     return {Status::NotOK, "Expect a Shebang statement in the first line"};
   }
 
-  static constexpr const char *shebang_prefix = "#!lua";
-  static constexpr const char *shebang_libname_prefix = "name=";
+  const auto libname = GET_OR_RET(ExtractLibNameFromShebang(first_line));
 
-  auto first_line_split = util::Split(first_line, " \r\t");
-  if (first_line_split.empty() || first_line_split[0] != shebang_prefix) {
-    return {Status::NotOK, "Expect a Shebang statement in the first line, e.g. `#!lua name=mylib`"};
-  }
-
-  size_t libname_pos = 1;
-  for (; libname_pos < first_line_split.size(); ++libname_pos) {
-    if (util::HasPrefix(first_line_split[libname_pos], shebang_libname_prefix)) {
-      break;
-    }
-  }
-
-  if (libname_pos >= first_line_split.size()) {
-    return {Status::NotOK, "Expect library name in the Shebang statement, e.g. `#!lua name=mylib`"};
-  }
-
-  auto libname = first_line_split[libname_pos].substr(strlen(shebang_libname_prefix));
-  *lib_name = libname;
-  if (libname.empty() ||
-      std::any_of(libname.begin(), libname.end(), [](char v) { return !std::isalnum(v) && v != '_'; })) {
-    return {Status::NotOK, "Expect a valid library name in the Shebang statement"};
-  }
   auto srv = conn->GetServer();
   auto lua = read_only ? conn->Owner()->Lua() : srv->Lua();
 
@@ -320,8 +300,8 @@ Status FunctionLoad(redis::Connection *conn, const std::string &script, bool nee
     if (!replace) {
       return {Status::NotOK, "library already exists, please specify REPLACE to force load"};
     }
-
-    auto s = FunctionDelete(srv, libname);
+    engine::Context ctx(srv->storage);
+    auto s = FunctionDelete(ctx, srv, libname);
     if (!s) return s;
   }
 
@@ -409,13 +389,24 @@ Status FunctionCall(redis::Connection *conn, const std::string &name, const std:
     std::string libcode;
     s = srv->FunctionGetCode(libname, &libcode);
     if (!s) return s;
-
     s = FunctionLoad(conn, libcode, false, false, &libname, read_only);
     if (!s) return s;
 
     lua_getglobal(lua, (REDIS_LUA_REGISTER_FUNC_PREFIX + name).c_str());
   }
 
+  ScriptRunCtx script_run_ctx;
+  script_run_ctx.flags = read_only ? ScriptFlagType::kScriptNoWrites : 0;
+  lua_getglobal(lua, (REDIS_LUA_REGISTER_FUNC_FLAGS_PREFIX + name).c_str());
+  if (!lua_isnil(lua, -1)) {
+    // It should be ensured that the conversion is successful
+    auto function_flags = lua_tointeger(lua, -1);
+    script_run_ctx.flags |= function_flags;
+  }
+  lua_pop(lua, 1);
+
+  SaveOnRegistry(lua, REGISTRY_SCRIPT_RUN_CTX_NAME, &script_run_ctx);
+
   PushArray(lua, keys);
   PushArray(lua, argv);
   if (lua_pcall(lua, 2, 1, -4)) {
@@ -427,6 +418,22 @@ Status FunctionCall(redis::Connection *conn, const std::string &name, const std:
     lua_pop(lua, 2);
   }
 
+  RemoveFromRegistry(lua, REGISTRY_SCRIPT_RUN_CTX_NAME);
+
+  /* Call the Lua garbage collector from time to time to avoid a
+   * full cycle performed by Lua, which adds too latency.
+   *
+   * The call is performed every LUA_GC_CYCLE_PERIOD executed commands
+   * (and for LUA_GC_CYCLE_PERIOD collection steps) because calling it
+   * for every command uses too much CPU. */
+  constexpr int64_t LUA_GC_CYCLE_PERIOD = 50;
+  static int64_t gc_count = 0;
+
+  gc_count++;
+  if (gc_count == LUA_GC_CYCLE_PERIOD) {
+    lua_gc(lua, LUA_GCSTEP, LUA_GC_CYCLE_PERIOD);
+    gc_count = 0;
+  }
   return Status::OK();
 }
 
@@ -437,14 +444,13 @@ Status FunctionList(Server *srv, const redis::Connection *conn, const std::strin
   std::string end_key = start_key;
   end_key.back()++;
 
-  rocksdb::ReadOptions read_options = srv->storage->DefaultScanOptions();
-  redis::LatestSnapShot ss(srv->storage);
-  read_options.snapshot = ss.GetSnapShot();
+  engine::Context ctx(srv->storage);
+  rocksdb::ReadOptions read_options = ctx.DefaultScanOptions();
   rocksdb::Slice upper_bound(end_key);
   read_options.iterate_upper_bound = &upper_bound;
 
   auto *cf = srv->storage->GetCFHandle(ColumnFamilyID::Propagate);
-  auto iter = util::UniqueIterator(srv->storage, read_options, cf);
+  auto iter = util::UniqueIterator(ctx, read_options, cf);
   std::vector<std::pair<std::string, std::string>> result;
   for (iter->Seek(start_key); iter->Valid(); iter->Next()) {
     Slice lib = iter->key();
@@ -452,6 +458,10 @@ Status FunctionList(Server *srv, const redis::Connection *conn, const std::strin
     result.emplace_back(lib.ToString(), iter->value().ToString());
   }
 
+  if (auto s = iter->status(); !s.ok()) {
+    return {Status::NotOK, s.ToString()};
+  }
+
   output->append(redis::MultiLen(result.size()));
   for (const auto &[lib, code] : result) {
     output->append(conn->HeaderOfMap(with_code ? 2 : 1));
@@ -473,14 +483,13 @@ Status FunctionListFunc(Server *srv, const redis::Connection *conn, const std::s
   std::string end_key = start_key;
   end_key.back()++;
 
-  rocksdb::ReadOptions read_options = srv->storage->DefaultScanOptions();
-  redis::LatestSnapShot ss(srv->storage);
-  read_options.snapshot = ss.GetSnapShot();
+  engine::Context ctx(srv->storage);
+  rocksdb::ReadOptions read_options = ctx.DefaultScanOptions();
   rocksdb::Slice upper_bound(end_key);
   read_options.iterate_upper_bound = &upper_bound;
 
   auto *cf = srv->storage->GetCFHandle(ColumnFamilyID::Propagate);
-  auto iter = util::UniqueIterator(srv->storage, read_options, cf);
+  auto iter = util::UniqueIterator(ctx, read_options, cf);
   std::vector<std::pair<std::string, std::string>> result;
   for (iter->Seek(start_key); iter->Valid(); iter->Next()) {
     Slice func = iter->key();
@@ -488,6 +497,10 @@ Status FunctionListFunc(Server *srv, const redis::Connection *conn, const std::s
     result.emplace_back(func.ToString(), iter->value().ToString());
   }
 
+  if (auto s = iter->status(); !s.ok()) {
+    return {Status::NotOK, s.ToString()};
+  }
+
   output->append(redis::MultiLen(result.size()));
   for (const auto &[func, lib] : result) {
     output->append(conn->HeaderOfMap(2));
@@ -541,7 +554,7 @@ Status FunctionListLib(Server *srv, const redis::Connection *conn, const std::st
   return Status::OK();
 }
 
-Status FunctionDelete(Server *srv, const std::string &name) {
+Status FunctionDelete(engine::Context &ctx, Server *srv, const std::string &name) {
   auto lua = srv->Lua();
 
   lua_getglobal(lua, REDIS_FUNCTION_LIBRARIES);
@@ -564,7 +577,9 @@ Status FunctionDelete(Server *srv, const std::string &name) {
     std::string func = lua_tostring(lua, -1);
     lua_pushnil(lua);
     lua_setglobal(lua, (REDIS_LUA_REGISTER_FUNC_PREFIX + func).c_str());
-    auto _ = storage->Delete(rocksdb::WriteOptions(), cf, engine::kLuaFuncLibPrefix + func);
+    lua_pushnil(lua);
+    lua_setglobal(lua, (REDIS_LUA_REGISTER_FUNC_FLAGS_PREFIX + func).c_str());
+    auto _ = storage->Delete(ctx, rocksdb::WriteOptions(), cf, engine::kLuaFuncLibPrefix + func);
     lua_pop(lua, 1);
   }
 
@@ -573,7 +588,7 @@ Status FunctionDelete(Server *srv, const std::string &name) {
   lua_setfield(lua, -2, name.c_str());
   lua_pop(lua, 1);
 
-  auto s = storage->Delete(rocksdb::WriteOptions(), cf, engine::kLuaLibCodePrefix + name);
+  auto s = storage->Delete(ctx, rocksdb::WriteOptions(), cf, engine::kLuaLibCodePrefix + name);
   if (!s.ok()) return {Status::NotOK, s.ToString()};
 
   return Status::OK();
@@ -582,7 +597,6 @@ Status FunctionDelete(Server *srv, const std::string &name) {
 Status EvalGenericCommand(redis::Connection *conn, const std::string &body_or_sha, const std::vector<std::string> &keys,
                           const std::vector<std::string> &argv, bool evalsha, std::string *output, bool read_only) {
   Server *srv = conn->GetServer();
-
   // Use the worker's private Lua VM when entering the read-only mode
   lua_State *lua = read_only ? conn->Owner()->Lua() : srv->Lua();
 
@@ -627,6 +641,18 @@ Status EvalGenericCommand(redis::Connection *conn, const std::string &body_or_sh
     lua_getglobal(lua, funcname);
   }
 
+  ScriptRunCtx current_script_run_ctx;
+  current_script_run_ctx.flags = read_only ? ScriptFlagType::kScriptNoWrites : 0;
+  lua_getglobal(lua, fmt::format(REDIS_LUA_FUNC_SHA_FLAGS, funcname + 2).c_str());
+  if (!lua_isnil(lua, -1)) {
+    // It should be ensured that the conversion is successful
+    auto script_flags = lua_tointeger(lua, -1);
+    current_script_run_ctx.flags |= script_flags;
+  }
+  lua_pop(lua, 1);
+
+  SaveOnRegistry(lua, REGISTRY_SCRIPT_RUN_CTX_NAME, &current_script_run_ctx);
+
   // For the Lua script, should be always run with RESP2 protocol,
   // unless users explicitly set the protocol version in the script via `redis.setresp`.
   // So we need to save the current protocol version and set it to RESP2,
@@ -653,6 +679,7 @@ Status EvalGenericCommand(redis::Connection *conn, const std::string &body_or_sh
   lua_setglobal(lua, "KEYS");
   lua_pushnil(lua);
   lua_setglobal(lua, "ARGV");
+  RemoveFromRegistry(lua, REGISTRY_SCRIPT_RUN_CTX_NAME);
 
   /* Call the Lua garbage collector from time to time to avoid a
    * full cycle performed by Lua, which adds too latency.
@@ -693,10 +720,8 @@ Server *GetServer(lua_State *lua) {
 // TODO: we do not want to repeat same logic as Connection::ExecuteCommands,
 // so the function need to be refactored
 int RedisGenericCommand(lua_State *lua, int raise_error) {
-  lua_getglobal(lua, "redis");
-  lua_getfield(lua, -1, "read_only");
-  int read_only = lua_toboolean(lua, -1);
-  lua_pop(lua, 2);
+  auto *script_run_ctx = GetFromRegistry<ScriptRunCtx>(lua, REGISTRY_SCRIPT_RUN_CTX_NAME);
+  CHECK_NOTNULL(script_run_ctx);
 
   int argc = lua_gettop(lua);
   if (argc == 0) {
@@ -730,7 +755,7 @@ int RedisGenericCommand(lua_State *lua, int raise_error) {
   auto attributes = cmd->GetAttributes();
   auto cmd_flags = attributes->GenerateFlags(args);
 
-  if (read_only && !(cmd_flags & redis::kCmdReadOnly)) {
+  if ((script_run_ctx->flags & ScriptFlagType::kScriptNoWrites) && !(cmd_flags & redis::kCmdReadOnly)) {
     PushError(lua, "Write commands are not allowed from read-only scripts");
     return raise_error ? RaiseError(lua) : 1;
   }
@@ -752,9 +777,17 @@ int RedisGenericCommand(lua_State *lua, int raise_error) {
 
   redis::Connection *conn = srv->GetCurrentConnection();
   if (config->cluster_enabled) {
-    auto s = srv->cluster->CanExecByMySelf(attributes, args, conn);
+    if (script_run_ctx->flags & ScriptFlagType::kScriptNoCluster) {
+      PushError(lua, "Can not run script on cluster, 'no-cluster' flag is set");
+      return raise_error ? RaiseError(lua) : 1;
+    }
+    auto s = srv->cluster->CanExecByMySelf(attributes, args, conn, script_run_ctx);
     if (!s.IsOK()) {
-      PushError(lua, redis::StatusToRedisErrorMsg(s).c_str());
+      if (s.Is<Status::RedisMoved>()) {
+        PushError(lua, "Script attempted to access a non local key in a cluster node script");
+      } else {
+        PushError(lua, redis::StatusToRedisErrorMsg(s).c_str());
+      }
       return raise_error ? RaiseError(lua) : 1;
     }
   }
@@ -1455,7 +1488,23 @@ Status CreateFunction(Server *srv, const std::string &body, std::string *sha, lu
     std::copy(sha->begin(), sha->end(), funcname + 2);
   }
 
-  if (luaL_loadbuffer(lua, body.c_str(), body.size(), "@user_script")) {
+  std::string_view lua_code(body);
+  // Cache the flags of the current script
+  ScriptFlags script_flags = 0;
+  if (auto pos = body.find('\n'); pos != std::string::npos) {
+    std::string_view first_line(body.data(), pos);
+    if (first_line.substr(0, 2) == "#!") {
+      lua_code = lua_code.substr(pos + 1);
+    }
+    script_flags = GET_OR_RET(ExtractFlagsFromShebang(first_line));
+  } else {
+    // scripts without #! can run commands that access keys belonging to different cluster hash slots
+    script_flags = kScriptAllowCrossSlotKeys;
+  }
+  lua_pushinteger(lua, static_cast<lua_Integer>(script_flags));
+  lua_setglobal(lua, fmt::format(REDIS_LUA_FUNC_SHA_FLAGS, *sha).c_str());
+
+  if (luaL_loadbuffer(lua, lua_code.data(), lua_code.size(), "@user_script")) {
     std::string err_msg = lua_tostring(lua, -1);
     lua_pop(lua, 1);
     return {Status::NotOK, "Error while compiling new script: " + err_msg};
@@ -1466,4 +1515,123 @@ Status CreateFunction(Server *srv, const std::string &body, std::string *sha, lu
   return need_to_store ? srv->ScriptSet(*sha, body) : Status::OK();
 }
 
+[[nodiscard]] StatusOr<std::string> ExtractLibNameFromShebang(std::string_view shebang) {
+  static constexpr std::string_view lua_shebang_prefix = "#!lua";
+  static constexpr std::string_view shebang_libname_prefix = "name=";
+
+  if (shebang.substr(0, 2) != "#!") {
+    return {Status::NotOK, "Missing library meta"};
+  }
+
+  auto shebang_splits = util::Split(shebang, " ");
+  if (shebang_splits.empty() || shebang_splits[0] != lua_shebang_prefix) {
+    return {Status::NotOK, "Unexpected engine in script shebang: " + shebang_splits[0]};
+  }
+
+  std::string libname;
+  bool found_libname = false;
+  for (size_t i = 1; i < shebang_splits.size(); i++) {
+    std::string_view shebang_split_sv = shebang_splits[i];
+    if (shebang_split_sv.substr(0, shebang_libname_prefix.size()) != shebang_libname_prefix) {
+      return {Status::NotOK, "Unknown lua shebang option: " + shebang_splits[i]};
+    }
+    if (found_libname) {
+      return {Status::NotOK, "Redundant library name in script shebang"};
+    }
+
+    libname = shebang_split_sv.substr(shebang_libname_prefix.size());
+    if (libname.empty() ||
+        std::any_of(libname.begin(), libname.end(), [](char v) { return !std::isalnum(v) && v != '_'; })) {
+      return {
+          Status::NotOK,
+          "Library names can only contain letters, numbers, or underscores(_) and must be at least one character long"};
+    }
+    found_libname = true;
+  }
+
+  if (found_libname) return libname;
+  return {Status::NotOK, "Library name was not given"};
+}
+
+[[nodiscard]] StatusOr<ScriptFlags> GetFlagsFromStrings(const std::vector<std::string> &flags_content) {
+  ScriptFlags flags = 0;
+  for (const auto &flag : flags_content) {
+    if (flag == "no-writes") {
+      flags |= kScriptNoWrites;
+    } else if (flag == "allow-oom") {
+      return {Status::NotSupported, "allow-oom is not supported yet"};
+    } else if (flag == "allow-stale") {
+      return {Status::NotSupported, "allow-stale is not supported yet"};
+    } else if (flag == "no-cluster") {
+      flags |= kScriptNoCluster;
+    } else if (flag == "allow-cross-slot-keys") {
+      flags |= kScriptAllowCrossSlotKeys;
+    } else {
+      return {Status::NotOK, "Unknown flag given: " + flag};
+    }
+  }
+  return flags;
+}
+
+[[nodiscard]] StatusOr<ScriptFlags> ExtractFlagsFromShebang(std::string_view shebang) {
+  static constexpr std::string_view lua_shebang_prefix = "#!lua";
+  static constexpr std::string_view shebang_flags_prefix = "flags=";
+
+  ScriptFlags result_flags = 0;
+  if (shebang.substr(0, 2) == "#!") {
+    auto shebang_splits = util::Split(shebang, " ");
+    if (shebang_splits.empty() || shebang_splits[0] != lua_shebang_prefix) {
+      return {Status::NotOK, "Unexpected engine in script shebang: " + shebang_splits[0]};
+    }
+    bool found_flags = false;
+    for (size_t i = 1; i < shebang_splits.size(); i++) {
+      std::string_view shebang_split_sv = shebang_splits[i];
+      if (shebang_split_sv.substr(0, shebang_flags_prefix.size()) != shebang_flags_prefix) {
+        return {Status::NotOK, "Unknown lua shebang option: " + shebang_splits[i]};
+      }
+      if (found_flags) {
+        return {Status::NotOK, "Redundant flags in script shebang"};
+      }
+      auto flags_content = util::Split(shebang_split_sv.substr(shebang_flags_prefix.size()), ",");
+      result_flags |= GET_OR_RET(GetFlagsFromStrings(flags_content));
+      found_flags = true;
+    }
+  } else {
+    // scripts without #! can run commands that access keys belonging to different cluster hash slots,
+    // but ones with #! inherit the default flags, so they cannot.
+    result_flags = kScriptAllowCrossSlotKeys;
+  }
+
+  return result_flags;
+}
+
+[[nodiscard]] StatusOr<ScriptFlags> ExtractFlagsFromRegisterFunction(lua_State *lua) {
+  if (!lua_istable(lua, -1)) {
+    return {Status::NotOK, "Expects a valid flags argument to register_function, e.g. flags={ 'no-writes' }"};
+  }
+  auto flag_count = static_cast<int>(lua_objlen(lua, -1));
+  std::vector<std::string> flags_content;
+  flags_content.reserve(flag_count);
+  for (int i = 1; i <= flag_count; ++i) {
+    lua_pushnumber(lua, i);
+    lua_gettable(lua, -2);
+    if (!lua_isstring(lua, -1)) {
+      return {Status::NotOK, "Expects a valid flags argument to register_function, e.g. flags={ 'no-writes' }"};
+    }
+    flags_content.emplace_back(lua_tostring(lua, -1));
+    // pop up the current flag
+    lua_pop(lua, 1);
+  }
+  // pop up the corresponding table of the flags parameter
+  lua_pop(lua, 1);
+
+  return GetFlagsFromStrings(flags_content);
+}
+
+void RemoveFromRegistry(lua_State *lua, const char *name) {
+  lua_pushstring(lua, name);
+  lua_pushnil(lua);
+  lua_settable(lua, LUA_REGISTRYINDEX);
+}
+
 }  // namespace lua
