@@ -27,31 +27,33 @@
 
 namespace redis {
 
-rocksdb::Status List::GetMetadata(Database::GetOptions get_options, const Slice &ns_key, ListMetadata *metadata) {
-  return Database::GetMetadata(get_options, {kRedisList}, ns_key, metadata);
+rocksdb::Status List::GetMetadata(engine::Context &ctx, const Slice &ns_key, ListMetadata *metadata) {
+  return Database::GetMetadata(ctx, {kRedisList}, ns_key, metadata);
 }
 
-rocksdb::Status List::Size(const Slice &user_key, uint64_t *size) {
+rocksdb::Status List::Size(engine::Context &ctx, const Slice &user_key, uint64_t *size) {
   *size = 0;
 
   std::string ns_key = AppendNamespacePrefix(user_key);
   ListMetadata metadata(false);
-  rocksdb::Status s = GetMetadata(GetOptions{}, ns_key, &metadata);
+  rocksdb::Status s = GetMetadata(ctx, ns_key, &metadata);
   if (!s.ok()) return s.IsNotFound() ? rocksdb::Status::OK() : s;
   *size = metadata.size;
   return rocksdb::Status::OK();
 }
 
-rocksdb::Status List::Push(const Slice &user_key, const std::vector<Slice> &elems, bool left, uint64_t *new_size) {
-  return push(user_key, elems, true, left, new_size);
-}
-
-rocksdb::Status List::PushX(const Slice &user_key, const std::vector<Slice> &elems, bool left, uint64_t *new_size) {
-  return push(user_key, elems, false, left, new_size);
-}
-
-rocksdb::Status List::push(const Slice &user_key, const std::vector<Slice> &elems, bool create_if_missing, bool left,
+rocksdb::Status List::Push(engine::Context &ctx, const Slice &user_key, const std::vector<Slice> &elems, bool left,
                            uint64_t *new_size) {
+  return push(ctx, user_key, elems, true, left, new_size);
+}
+
+rocksdb::Status List::PushX(engine::Context &ctx, const Slice &user_key, const std::vector<Slice> &elems, bool left,
+                            uint64_t *new_size) {
+  return push(ctx, user_key, elems, false, left, new_size);
+}
+
+rocksdb::Status List::push(engine::Context &ctx, const Slice &user_key, const std::vector<Slice> &elems,
+                           bool create_if_missing, bool left, uint64_t *new_size) {
   *new_size = 0;
   std::string ns_key = AppendNamespacePrefix(user_key);
 
@@ -59,9 +61,10 @@ rocksdb::Status List::push(const Slice &user_key, const std::vector<Slice> &elem
   auto batch = storage_->GetWriteBatchBase();
   RedisCommand cmd = left ? kRedisCmdLPush : kRedisCmdRPush;
   WriteBatchLogData log_data(kRedisList, {std::to_string(cmd)});
-  batch->PutLogData(log_data.Encode());
+  auto s = batch->PutLogData(log_data.Encode());
+  if (!s.ok()) return s;
   LockGuard guard(storage_->GetLockManager(), ns_key);
-  rocksdb::Status s = GetMetadata(GetOptions{}, ns_key, &metadata);
+  s = GetMetadata(ctx, ns_key, &metadata);
   if (!s.ok() && !(create_if_missing && s.IsNotFound())) {
     return s.IsNotFound() ? rocksdb::Status::OK() : s;
   }
@@ -70,7 +73,8 @@ rocksdb::Status List::push(const Slice &user_key, const std::vector<Slice> &elem
     std::string index_buf;
     PutFixed64(&index_buf, index);
     std::string sub_key = InternalKey(ns_key, index_buf, metadata.version, storage_->IsSlotIdEncoded()).Encode();
-    batch->Put(sub_key, elem);
+    s = batch->Put(sub_key, elem);
+    if (!s.ok()) return s;
     left ? --index : ++index;
   }
   if (left) {
@@ -81,23 +85,24 @@ rocksdb::Status List::push(const Slice &user_key, const std::vector<Slice> &elem
   std::string bytes;
   metadata.size += elems.size();
   metadata.Encode(&bytes);
-  batch->Put(metadata_cf_handle_, ns_key, bytes);
+  s = batch->Put(metadata_cf_handle_, ns_key, bytes);
+  if (!s.ok()) return s;
   *new_size = metadata.size;
-  return storage_->Write(storage_->DefaultWriteOptions(), batch->GetWriteBatch());
+  return storage_->Write(ctx, storage_->DefaultWriteOptions(), batch->GetWriteBatch());
 }
 
-rocksdb::Status List::Pop(const Slice &user_key, bool left, std::string *elem) {
+rocksdb::Status List::Pop(engine::Context &ctx, const Slice &user_key, bool left, std::string *elem) {
   elem->clear();
 
   std::vector<std::string> elems;
-  auto s = PopMulti(user_key, left, 1, &elems);
+  auto s = PopMulti(ctx, user_key, left, 1, &elems);
   if (!s.ok()) return s;
 
   *elem = std::move(elems[0]);
   return rocksdb::Status::OK();
 }
 
-rocksdb::Status List::PopMulti(const rocksdb::Slice &user_key, bool left, uint32_t count,
+rocksdb::Status List::PopMulti(engine::Context &ctx, const rocksdb::Slice &user_key, bool left, uint32_t count,
                                std::vector<std::string> *elems) {
   elems->clear();
 
@@ -105,13 +110,14 @@ rocksdb::Status List::PopMulti(const rocksdb::Slice &user_key, bool left, uint32
 
   LockGuard guard(storage_->GetLockManager(), ns_key);
   ListMetadata metadata(false);
-  rocksdb::Status s = GetMetadata(GetOptions{}, ns_key, &metadata);
+  rocksdb::Status s = GetMetadata(ctx, ns_key, &metadata);
   if (!s.ok()) return s;
 
   auto batch = storage_->GetWriteBatchBase();
   RedisCommand cmd = left ? kRedisCmdLPop : kRedisCmdRPop;
   WriteBatchLogData log_data(kRedisList, {std::to_string(cmd)});
-  batch->PutLogData(log_data.Encode());
+  s = batch->PutLogData(log_data.Encode());
+  if (!s.ok()) return s;
 
   while (metadata.size > 0 && count > 0) {
     uint64_t index = left ? metadata.head : metadata.tail - 1;
@@ -119,28 +125,31 @@ rocksdb::Status List::PopMulti(const rocksdb::Slice &user_key, bool left, uint32
     PutFixed64(&buf, index);
     std::string sub_key = InternalKey(ns_key, buf, metadata.version, storage_->IsSlotIdEncoded()).Encode();
     std::string elem;
-    s = storage_->Get(rocksdb::ReadOptions(), sub_key, &elem);
+    s = storage_->Get(ctx, ctx.GetReadOptions(), sub_key, &elem);
     if (!s.ok()) {
       // FIXME: should be always exists??
       return s;
     }
 
     elems->push_back(elem);
-    batch->Delete(sub_key);
+    s = batch->Delete(sub_key);
+    if (!s.ok()) return s;
     metadata.size -= 1;
     left ? ++metadata.head : --metadata.tail;
     --count;
   }
 
   if (metadata.size == 0) {
-    batch->Delete(metadata_cf_handle_, ns_key);
+    s = batch->Delete(metadata_cf_handle_, ns_key);
+    if (!s.ok()) return s;
   } else {
     std::string bytes;
     metadata.Encode(&bytes);
-    batch->Put(metadata_cf_handle_, ns_key, bytes);
+    s = batch->Put(metadata_cf_handle_, ns_key, bytes);
+    if (!s.ok()) return s;
   }
 
-  return storage_->Write(storage_->DefaultWriteOptions(), batch->GetWriteBatch());
+  return storage_->Write(ctx, storage_->DefaultWriteOptions(), batch->GetWriteBatch());
 }
 
 /*
@@ -162,14 +171,15 @@ rocksdb::Status List::PopMulti(const rocksdb::Slice &user_key, bool left, uint32
  * then trim the list from tail with num of elems to delete, here is 2.
  * and list would become: | E1 | E2 | E3 | E4 | E5 | E6 |
  */
-rocksdb::Status List::Rem(const Slice &user_key, int count, const Slice &elem, uint64_t *removed_cnt) {
+rocksdb::Status List::Rem(engine::Context &ctx, const Slice &user_key, int count, const Slice &elem,
+                          uint64_t *removed_cnt) {
   *removed_cnt = 0;
 
   std::string ns_key = AppendNamespacePrefix(user_key);
 
   LockGuard guard(storage_->GetLockManager(), ns_key);
   ListMetadata metadata(false);
-  rocksdb::Status s = GetMetadata(GetOptions{}, ns_key, &metadata);
+  rocksdb::Status s = GetMetadata(ctx, ns_key, &metadata);
   if (!s.ok()) return s;
 
   uint64_t index = count >= 0 ? metadata.head : metadata.tail - 1;
@@ -181,15 +191,13 @@ rocksdb::Status List::Rem(const Slice &user_key, int count, const Slice &elem, u
 
   bool reversed = count < 0;
   std::vector<uint64_t> to_delete_indexes;
-  rocksdb::ReadOptions read_options = storage_->DefaultScanOptions();
-  LatestSnapShot ss(storage_);
-  read_options.snapshot = ss.GetSnapShot();
+  rocksdb::ReadOptions read_options = ctx.DefaultScanOptions();
   rocksdb::Slice upper_bound(next_version_prefix);
   read_options.iterate_upper_bound = &upper_bound;
   rocksdb::Slice lower_bound(prefix);
   read_options.iterate_lower_bound = &lower_bound;
 
-  auto iter = util::UniqueIterator(storage_, read_options);
+  auto iter = util::UniqueIterator(ctx, read_options);
   for (iter->Seek(start_key); iter->Valid() && iter->key().starts_with(prefix);
        !reversed ? iter->Next() : iter->Prev()) {
     if (iter->value() == elem) {
@@ -206,10 +214,12 @@ rocksdb::Status List::Rem(const Slice &user_key, int count, const Slice &elem, u
 
   auto batch = storage_->GetWriteBatchBase();
   WriteBatchLogData log_data(kRedisList, {std::to_string(kRedisCmdLRem), std::to_string(count), elem.ToString()});
-  batch->PutLogData(log_data.Encode());
+  s = batch->PutLogData(log_data.Encode());
+  if (!s.ok()) return s;
 
   if (to_delete_indexes.size() == metadata.size) {
-    batch->Delete(metadata_cf_handle_, ns_key);
+    s = batch->Delete(metadata_cf_handle_, ns_key);
+    if (!s.ok()) return s;
   } else {
     uint64_t min_to_delete_index = !reversed ? to_delete_indexes[0] : to_delete_indexes[to_delete_indexes.size() - 1];
     uint64_t max_to_delete_index = !reversed ? to_delete_indexes[to_delete_indexes.size() - 1] : to_delete_indexes[0];
@@ -226,7 +236,8 @@ rocksdb::Status List::Rem(const Slice &user_key, int count, const Slice &elem, u
         buf.clear();
         PutFixed64(&buf, reversed ? max_to_delete_index-- : min_to_delete_index++);
         std::string to_update_key = InternalKey(ns_key, buf, metadata.version, storage_->IsSlotIdEncoded()).Encode();
-        batch->Put(to_update_key, iter->value());
+        s = batch->Put(to_update_key, iter->value());
+        if (!s.ok()) return s;
       } else {
         processed++;
       }
@@ -236,7 +247,8 @@ rocksdb::Status List::Rem(const Slice &user_key, int count, const Slice &elem, u
       buf.clear();
       PutFixed64(&buf, reversed ? (metadata.head + idx) : (metadata.tail - 1 - idx));
       std::string to_delete_key = InternalKey(ns_key, buf, metadata.version, storage_->IsSlotIdEncoded()).Encode();
-      batch->Delete(to_delete_key);
+      s = batch->Delete(to_delete_key);
+      if (!s.ok()) return s;
     }
     if (reversed) {
       metadata.head += to_delete_indexes.size();
@@ -246,20 +258,22 @@ rocksdb::Status List::Rem(const Slice &user_key, int count, const Slice &elem, u
     metadata.size -= to_delete_indexes.size();
     std::string bytes;
     metadata.Encode(&bytes);
-    batch->Put(metadata_cf_handle_, ns_key, bytes);
+    s = batch->Put(metadata_cf_handle_, ns_key, bytes);
+    if (!s.ok()) return s;
   }
 
   *removed_cnt = to_delete_indexes.size();
-  return storage_->Write(storage_->DefaultWriteOptions(), batch->GetWriteBatch());
+  return storage_->Write(ctx, storage_->DefaultWriteOptions(), batch->GetWriteBatch());
 }
 
-rocksdb::Status List::Insert(const Slice &user_key, const Slice &pivot, const Slice &elem, bool before, int *new_size) {
+rocksdb::Status List::Insert(engine::Context &ctx, const Slice &user_key, const Slice &pivot, const Slice &elem,
+                             bool before, int *new_size) {
   *new_size = 0;
   std::string ns_key = AppendNamespacePrefix(user_key);
 
   LockGuard guard(storage_->GetLockManager(), ns_key);
   ListMetadata metadata(false);
-  rocksdb::Status s = GetMetadata(GetOptions{}, ns_key, &metadata);
+  rocksdb::Status s = GetMetadata(ctx, ns_key, &metadata);
   if (!s.ok()) return s;
 
   std::string buf;
@@ -269,13 +283,11 @@ rocksdb::Status List::Insert(const Slice &user_key, const Slice &pivot, const Sl
   std::string prefix = InternalKey(ns_key, "", metadata.version, storage_->IsSlotIdEncoded()).Encode();
   std::string next_version_prefix = InternalKey(ns_key, "", metadata.version + 1, storage_->IsSlotIdEncoded()).Encode();
 
-  rocksdb::ReadOptions read_options = storage_->DefaultScanOptions();
-  LatestSnapShot ss(storage_);
-  read_options.snapshot = ss.GetSnapShot();
+  rocksdb::ReadOptions read_options = ctx.DefaultScanOptions();
   rocksdb::Slice upper_bound(next_version_prefix);
   read_options.iterate_upper_bound = &upper_bound;
 
-  auto iter = util::UniqueIterator(storage_, read_options);
+  auto iter = util::UniqueIterator(ctx, read_options);
   for (iter->Seek(start_key); iter->Valid() && iter->key().starts_with(prefix); iter->Next()) {
     if (iter->value() == pivot) {
       InternalKey ikey(iter->key(), storage_->IsSlotIdEncoded());
@@ -292,7 +304,8 @@ rocksdb::Status List::Insert(const Slice &user_key, const Slice &pivot, const Sl
   auto batch = storage_->GetWriteBatchBase();
   WriteBatchLogData log_data(kRedisList,
                              {std::to_string(kRedisCmdLInsert), before ? "1" : "0", pivot.ToString(), elem.ToString()});
-  batch->PutLogData(log_data.Encode());
+  s = batch->PutLogData(log_data.Encode());
+  if (!s.ok()) return s;
 
   uint64_t left_part_len = pivot_index - metadata.head + (before ? 0 : 1);
   uint64_t right_part_len = metadata.tail - 1 - pivot_index + (before ? 1 : 0);
@@ -308,12 +321,14 @@ rocksdb::Status List::Insert(const Slice &user_key, const Slice &pivot, const Sl
     buf.clear();
     PutFixed64(&buf, reversed ? --pivot_index : ++pivot_index);
     std::string to_update_key = InternalKey(ns_key, buf, metadata.version, storage_->IsSlotIdEncoded()).Encode();
-    batch->Put(to_update_key, iter->value());
+    s = batch->Put(to_update_key, iter->value());
+    if (!s.ok()) return s;
   }
   buf.clear();
   PutFixed64(&buf, new_elem_index);
   std::string to_update_key = InternalKey(ns_key, buf, metadata.version, storage_->IsSlotIdEncoded()).Encode();
-  batch->Put(to_update_key, elem);
+  s = batch->Put(to_update_key, elem);
+  if (!s.ok()) return s;
 
   if (reversed) {
     metadata.head--;
@@ -323,30 +338,28 @@ rocksdb::Status List::Insert(const Slice &user_key, const Slice &pivot, const Sl
   metadata.size++;
   std::string bytes;
   metadata.Encode(&bytes);
-  batch->Put(metadata_cf_handle_, ns_key, bytes);
+  s = batch->Put(metadata_cf_handle_, ns_key, bytes);
+  if (!s.ok()) return s;
 
   *new_size = static_cast<int>(metadata.size);
-  return storage_->Write(storage_->DefaultWriteOptions(), batch->GetWriteBatch());
+  return storage_->Write(ctx, storage_->DefaultWriteOptions(), batch->GetWriteBatch());
 }
 
-rocksdb::Status List::Index(const Slice &user_key, int index, std::string *elem) {
+rocksdb::Status List::Index(engine::Context &ctx, const Slice &user_key, int index, std::string *elem) {
   elem->clear();
 
   std::string ns_key = AppendNamespacePrefix(user_key);
   ListMetadata metadata(false);
-  LatestSnapShot ss(storage_);
-  rocksdb::Status s = GetMetadata(GetOptions{ss.GetSnapShot()}, ns_key, &metadata);
+  rocksdb::Status s = GetMetadata(ctx, ns_key, &metadata);
   if (!s.ok()) return s;
 
   if (index < 0) index += static_cast<int>(metadata.size);
   if (index < 0 || index >= static_cast<int>(metadata.size)) return rocksdb::Status::NotFound();
 
-  rocksdb::ReadOptions read_options;
-  read_options.snapshot = ss.GetSnapShot();
   std::string buf;
   PutFixed64(&buf, metadata.head + index);
   std::string sub_key = InternalKey(ns_key, buf, metadata.version, storage_->IsSlotIdEncoded()).Encode();
-  return storage_->Get(read_options, sub_key, elem);
+  return storage_->Get(ctx, ctx.GetReadOptions(), sub_key, elem);
 }
 
 // The offset can also be negative, -1 is the last element, -2 the penultimate
@@ -354,13 +367,13 @@ rocksdb::Status List::Index(const Slice &user_key, int index, std::string *elem)
 // If start is larger than the end of the list, an empty list is returned.
 // If stop is larger than the actual end of the list,
 // Redis will treat it like the last element of the list.
-rocksdb::Status List::Range(const Slice &user_key, int start, int stop, std::vector<std::string> *elems) {
+rocksdb::Status List::Range(engine::Context &ctx, const Slice &user_key, int start, int stop,
+                            std::vector<std::string> *elems) {
   elems->clear();
 
   std::string ns_key = AppendNamespacePrefix(user_key);
   ListMetadata metadata(false);
-  LatestSnapShot ss(storage_);
-  rocksdb::Status s = GetMetadata(GetOptions{ss.GetSnapShot()}, ns_key, &metadata);
+  rocksdb::Status s = GetMetadata(ctx, ns_key, &metadata);
   if (!s.ok()) return s.IsNotFound() ? rocksdb::Status::OK() : s;
 
   if (start < 0) start = static_cast<int>(metadata.size) + start;
@@ -374,12 +387,11 @@ rocksdb::Status List::Range(const Slice &user_key, int start, int stop, std::vec
   std::string prefix = InternalKey(ns_key, "", metadata.version, storage_->IsSlotIdEncoded()).Encode();
   std::string next_version_prefix = InternalKey(ns_key, "", metadata.version + 1, storage_->IsSlotIdEncoded()).Encode();
 
-  rocksdb::ReadOptions read_options = storage_->DefaultScanOptions();
-  read_options.snapshot = ss.GetSnapShot();
+  rocksdb::ReadOptions read_options = ctx.DefaultScanOptions();
   rocksdb::Slice upper_bound(next_version_prefix);
   read_options.iterate_upper_bound = &upper_bound;
 
-  auto iter = util::UniqueIterator(storage_, read_options);
+  auto iter = util::UniqueIterator(ctx, read_options);
   for (iter->Seek(start_key); iter->Valid() && iter->key().starts_with(prefix); iter->Next()) {
     InternalKey ikey(iter->key(), storage_->IsSlotIdEncoded());
     Slice sub_key = ikey.GetSubKey();
@@ -392,13 +404,13 @@ rocksdb::Status List::Range(const Slice &user_key, int start, int stop, std::vec
   return rocksdb::Status::OK();
 }
 
-rocksdb::Status List::Pos(const Slice &user_key, const Slice &elem, const PosSpec &spec,
+rocksdb::Status List::Pos(engine::Context &ctx, const Slice &user_key, const Slice &elem, const PosSpec &spec,
                           std::vector<int64_t> *indexes) {
   indexes->clear();
 
   std::string ns_key = AppendNamespacePrefix(user_key);
   ListMetadata metadata(false);
-  rocksdb::Status s = GetMetadata(GetOptions{}, ns_key, &metadata);
+  rocksdb::Status s = GetMetadata(ctx, ns_key, &metadata);
   if (!s.ok()) return s;
 
   // A negative rank means start from the tail.
@@ -417,9 +429,7 @@ rocksdb::Status List::Pos(const Slice &user_key, const Slice &elem, const PosSpe
   std::string prefix = InternalKey(ns_key, "", metadata.version, storage_->IsSlotIdEncoded()).Encode();
   std::string next_version_prefix = InternalKey(ns_key, "", metadata.version + 1, storage_->IsSlotIdEncoded()).Encode();
 
-  rocksdb::ReadOptions read_options = storage_->DefaultScanOptions();
-  LatestSnapShot ss(storage_);
-  read_options.snapshot = ss.GetSnapShot();
+  rocksdb::ReadOptions read_options = ctx.DefaultScanOptions();
   rocksdb::Slice upper_bound(next_version_prefix);
   read_options.iterate_upper_bound = &upper_bound;
   rocksdb::Slice lower_bound(prefix);
@@ -430,7 +440,7 @@ rocksdb::Status List::Pos(const Slice &user_key, const Slice &elem, const PosSpe
   int64_t count = spec.count.value_or(-1);
   int64_t offset = 0, matches = 0;
 
-  auto iter = util::UniqueIterator(storage_, read_options);
+  auto iter = util::UniqueIterator(ctx, read_options);
   iter->Seek(start_key);
   while (iter->Valid() && iter->key().starts_with(prefix) && (max_len == 0 || offset < max_len)) {
     if (iter->value() == elem) {
@@ -449,12 +459,12 @@ rocksdb::Status List::Pos(const Slice &user_key, const Slice &elem, const PosSpe
   return rocksdb::Status::OK();
 }
 
-rocksdb::Status List::Set(const Slice &user_key, int index, Slice elem) {
+rocksdb::Status List::Set(engine::Context &ctx, const Slice &user_key, int index, Slice elem) {
   std::string ns_key = AppendNamespacePrefix(user_key);
 
   LockGuard guard(storage_->GetLockManager(), ns_key);
   ListMetadata metadata(false);
-  rocksdb::Status s = GetMetadata(GetOptions{}, ns_key, &metadata);
+  rocksdb::Status s = GetMetadata(ctx, ns_key, &metadata);
   if (!s.ok()) return s;
   if (index < 0) index += static_cast<int>(metadata.size);
   if (index < 0 || index >= static_cast<int>(metadata.size)) {
@@ -464,7 +474,7 @@ rocksdb::Status List::Set(const Slice &user_key, int index, Slice elem) {
   std::string buf, value;
   PutFixed64(&buf, metadata.head + index);
   std::string sub_key = InternalKey(ns_key, buf, metadata.version, storage_->IsSlotIdEncoded()).Encode();
-  s = storage_->Get(rocksdb::ReadOptions(), sub_key, &value);
+  s = storage_->Get(ctx, ctx.GetReadOptions(), sub_key, &value);
   if (!s.ok()) {
     return s;
   }
@@ -472,25 +482,28 @@ rocksdb::Status List::Set(const Slice &user_key, int index, Slice elem) {
 
   auto batch = storage_->GetWriteBatchBase();
   WriteBatchLogData log_data(kRedisList, {std::to_string(kRedisCmdLSet), std::to_string(index)});
-  batch->PutLogData(log_data.Encode());
-  batch->Put(sub_key, elem);
-  return storage_->Write(storage_->DefaultWriteOptions(), batch->GetWriteBatch());
+  s = batch->PutLogData(log_data.Encode());
+  if (!s.ok()) return s;
+  s = batch->Put(sub_key, elem);
+  if (!s.ok()) return s;
+  return storage_->Write(ctx, storage_->DefaultWriteOptions(), batch->GetWriteBatch());
 }
 
-rocksdb::Status List::LMove(const rocksdb::Slice &src, const rocksdb::Slice &dst, bool src_left, bool dst_left,
-                            std::string *elem) {
+rocksdb::Status List::LMove(engine::Context &ctx, const rocksdb::Slice &src, const rocksdb::Slice &dst, bool src_left,
+                            bool dst_left, std::string *elem) {
   if (src == dst) {
-    return lmoveOnSingleList(src, src_left, dst_left, elem);
+    return lmoveOnSingleList(ctx, src, src_left, dst_left, elem);
   }
-  return lmoveOnTwoLists(src, dst, src_left, dst_left, elem);
+  return lmoveOnTwoLists(ctx, src, dst, src_left, dst_left, elem);
 }
 
-rocksdb::Status List::lmoveOnSingleList(const rocksdb::Slice &src, bool src_left, bool dst_left, std::string *elem) {
+rocksdb::Status List::lmoveOnSingleList(engine::Context &ctx, const rocksdb::Slice &src, bool src_left, bool dst_left,
+                                        std::string *elem) {
   std::string ns_key = AppendNamespacePrefix(src);
 
   LockGuard guard(storage_->GetLockManager(), ns_key);
   ListMetadata metadata(false);
-  rocksdb::Status s = GetMetadata(GetOptions{}, ns_key, &metadata);
+  rocksdb::Status s = GetMetadata(ctx, ns_key, &metadata);
   if (!s.ok()) {
     return s;
   }
@@ -502,7 +515,7 @@ rocksdb::Status List::lmoveOnSingleList(const rocksdb::Slice &src, bool src_left
   PutFixed64(&curr_index_buf, curr_index);
   std::string curr_sub_key =
       InternalKey(ns_key, curr_index_buf, metadata.version, storage_->IsSlotIdEncoded()).Encode();
-  s = storage_->Get(rocksdb::ReadOptions(), curr_sub_key, elem);
+  s = storage_->Get(ctx, ctx.GetReadOptions(), curr_sub_key, elem);
   if (!s.ok()) {
     return s;
   }
@@ -520,9 +533,11 @@ rocksdb::Status List::lmoveOnSingleList(const rocksdb::Slice &src, bool src_left
   auto batch = storage_->GetWriteBatchBase();
   WriteBatchLogData log_data(kRedisList, {std::to_string(kRedisCmdLMove), src.ToString(), src.ToString(),
                                           src_left ? "left" : "right", dst_left ? "left" : "right"});
-  batch->PutLogData(log_data.Encode());
+  s = batch->PutLogData(log_data.Encode());
+  if (!s.ok()) return s;
 
-  batch->Delete(curr_sub_key);
+  s = batch->Delete(curr_sub_key);
+  if (!s.ok()) return s;
 
   if (src_left) {
     ++metadata.head;
@@ -536,30 +551,32 @@ rocksdb::Status List::lmoveOnSingleList(const rocksdb::Slice &src, bool src_left
   std::string new_index_buf;
   PutFixed64(&new_index_buf, new_index);
   std::string new_sub_key = InternalKey(ns_key, new_index_buf, metadata.version, storage_->IsSlotIdEncoded()).Encode();
-  batch->Put(new_sub_key, *elem);
+  s = batch->Put(new_sub_key, *elem);
+  if (!s.ok()) return s;
 
   std::string bytes;
   metadata.Encode(&bytes);
-  batch->Put(metadata_cf_handle_, ns_key, bytes);
+  s = batch->Put(metadata_cf_handle_, ns_key, bytes);
+  if (!s.ok()) return s;
 
-  return storage_->Write(storage_->DefaultWriteOptions(), batch->GetWriteBatch());
+  return storage_->Write(ctx, storage_->DefaultWriteOptions(), batch->GetWriteBatch());
 }
 
-rocksdb::Status List::lmoveOnTwoLists(const rocksdb::Slice &src, const rocksdb::Slice &dst, bool src_left,
-                                      bool dst_left, std::string *elem) {
+rocksdb::Status List::lmoveOnTwoLists(engine::Context &ctx, const rocksdb::Slice &src, const rocksdb::Slice &dst,
+                                      bool src_left, bool dst_left, std::string *elem) {
   std::string src_ns_key = AppendNamespacePrefix(src);
   std::string dst_ns_key = AppendNamespacePrefix(dst);
 
   std::vector<std::string> lock_keys{src_ns_key, dst_ns_key};
   MultiLockGuard guard(storage_->GetLockManager(), lock_keys);
   ListMetadata src_metadata(false);
-  auto s = GetMetadata(GetOptions{}, src_ns_key, &src_metadata);
+  auto s = GetMetadata(ctx, src_ns_key, &src_metadata);
   if (!s.ok()) {
     return s;
   }
 
   ListMetadata dst_metadata(false);
-  s = GetMetadata(GetOptions{}, dst_ns_key, &dst_metadata);
+  s = GetMetadata(ctx, dst_ns_key, &dst_metadata);
   if (!s.ok() && !s.IsNotFound()) {
     return s;
   }
@@ -569,27 +586,31 @@ rocksdb::Status List::lmoveOnTwoLists(const rocksdb::Slice &src, const rocksdb::
   auto batch = storage_->GetWriteBatchBase();
   WriteBatchLogData log_data(kRedisList, {std::to_string(kRedisCmdLMove), src.ToString(), dst.ToString(),
                                           src_left ? "left" : "right", dst_left ? "left" : "right"});
-  batch->PutLogData(log_data.Encode());
+  s = batch->PutLogData(log_data.Encode());
+  if (!s.ok()) return s;
 
   uint64_t src_index = src_left ? src_metadata.head : src_metadata.tail - 1;
   std::string src_buf;
   PutFixed64(&src_buf, src_index);
   std::string src_sub_key =
       InternalKey(src_ns_key, src_buf, src_metadata.version, storage_->IsSlotIdEncoded()).Encode();
-  s = storage_->Get(rocksdb::ReadOptions(), src_sub_key, elem);
+  s = storage_->Get(ctx, ctx.GetReadOptions(), src_sub_key, elem);
   if (!s.ok()) {
     return s;
   }
 
-  batch->Delete(src_sub_key);
+  s = batch->Delete(src_sub_key);
+  if (!s.ok()) return s;
   if (src_metadata.size == 1) {
-    batch->Delete(metadata_cf_handle_, src_ns_key);
+    s = batch->Delete(metadata_cf_handle_, src_ns_key);
+    if (!s.ok()) return s;
   } else {
     std::string bytes;
     src_metadata.size -= 1;
     src_left ? ++src_metadata.head : --src_metadata.tail;
     src_metadata.Encode(&bytes);
-    batch->Put(metadata_cf_handle_, src_ns_key, bytes);
+    s = batch->Put(metadata_cf_handle_, src_ns_key, bytes);
+    if (!s.ok()) return s;
   }
 
   uint64_t dst_index = dst_left ? dst_metadata.head - 1 : dst_metadata.tail;
@@ -597,25 +618,27 @@ rocksdb::Status List::lmoveOnTwoLists(const rocksdb::Slice &src, const rocksdb::
   PutFixed64(&dst_buf, dst_index);
   std::string dst_sub_key =
       InternalKey(dst_ns_key, dst_buf, dst_metadata.version, storage_->IsSlotIdEncoded()).Encode();
-  batch->Put(dst_sub_key, *elem);
+  s = batch->Put(dst_sub_key, *elem);
+  if (!s.ok()) return s;
   dst_left ? --dst_metadata.head : ++dst_metadata.tail;
 
   std::string bytes;
   dst_metadata.size += 1;
   dst_metadata.Encode(&bytes);
-  batch->Put(metadata_cf_handle_, dst_ns_key, bytes);
+  s = batch->Put(metadata_cf_handle_, dst_ns_key, bytes);
+  if (!s.ok()) return s;
 
-  return storage_->Write(storage_->DefaultWriteOptions(), batch->GetWriteBatch());
+  return storage_->Write(ctx, storage_->DefaultWriteOptions(), batch->GetWriteBatch());
 }
 
 // Caution: trim the big list may block the server
-rocksdb::Status List::Trim(const Slice &user_key, int start, int stop) {
+rocksdb::Status List::Trim(engine::Context &ctx, const Slice &user_key, int start, int stop) {
   uint32_t trim_cnt = 0;
   std::string ns_key = AppendNamespacePrefix(user_key);
 
   LockGuard guard(storage_->GetLockManager(), ns_key);
   ListMetadata metadata(false);
-  rocksdb::Status s = GetMetadata(GetOptions{}, ns_key, &metadata);
+  rocksdb::Status s = GetMetadata(ctx, ns_key, &metadata);
   if (!s.ok()) return s.IsNotFound() ? rocksdb::Status::OK() : s;
 
   if (start < 0) start += static_cast<int>(metadata.size);
@@ -623,21 +646,23 @@ rocksdb::Status List::Trim(const Slice &user_key, int start, int stop) {
   // the result will be empty list when start > stop,
   // or start is larger than the end of list
   if (start > stop) {
-    return storage_->Delete(storage_->DefaultWriteOptions(), metadata_cf_handle_, ns_key);
+    return storage_->Delete(ctx, storage_->DefaultWriteOptions(), metadata_cf_handle_, ns_key);
   }
   if (start < 0) start = 0;
 
   auto batch = storage_->GetWriteBatchBase();
   WriteBatchLogData log_data(kRedisList, std::vector<std::string>{std::to_string(kRedisCmdLTrim), std::to_string(start),
                                                                   std::to_string(stop)});
-  batch->PutLogData(log_data.Encode());
+  s = batch->PutLogData(log_data.Encode());
+  if (!s.ok()) return s;
   uint64_t left_index = metadata.head + start;
   uint64_t right_index = metadata.head + stop + 1;
   for (uint64_t i = metadata.head; i < left_index; i++) {
     std::string buf;
     PutFixed64(&buf, i);
     std::string sub_key = InternalKey(ns_key, buf, metadata.version, storage_->IsSlotIdEncoded()).Encode();
-    batch->Delete(sub_key);
+    s = batch->Delete(sub_key);
+    if (!s.ok()) return s;
     metadata.head++;
     trim_cnt++;
   }
@@ -646,7 +671,8 @@ rocksdb::Status List::Trim(const Slice &user_key, int start, int stop) {
     std::string buf;
     PutFixed64(&buf, i);
     std::string sub_key = InternalKey(ns_key, buf, metadata.version, storage_->IsSlotIdEncoded()).Encode();
-    batch->Delete(sub_key);
+    s = batch->Delete(sub_key);
+    if (!s.ok()) return s;
     metadata.tail--;
     trim_cnt++;
   }
@@ -657,7 +683,8 @@ rocksdb::Status List::Trim(const Slice &user_key, int start, int stop) {
   }
   std::string bytes;
   metadata.Encode(&bytes);
-  batch->Put(metadata_cf_handle_, ns_key, bytes);
-  return storage_->Write(storage_->DefaultWriteOptions(), batch->GetWriteBatch());
+  s = batch->Put(metadata_cf_handle_, ns_key, bytes);
+  if (!s.ok()) return s;
+  return storage_->Write(ctx, storage_->DefaultWriteOptions(), batch->GetWriteBatch());
 }
 }  // namespace redis
