@@ -27,23 +27,21 @@
 #include <string>
 
 #include "parse_util.h"
-#include "server/redis_request.h"
 #include "storage/redis_metadata.h"
 #include "time_util.h"
 
 namespace redis {
 
-std::vector<rocksdb::Status> String::getRawValues(const std::vector<Slice> &keys,
+std::vector<rocksdb::Status> String::getRawValues(engine::Context &ctx, const std::vector<Slice> &keys,
                                                   std::vector<std::string> *raw_values) {
   raw_values->clear();
 
-  rocksdb::ReadOptions read_options = storage_->DefaultMultiGetOptions();
-  LatestSnapShot ss(storage_);
-  read_options.snapshot = ss.GetSnapShot();
+  rocksdb::ReadOptions read_options = ctx.DefaultMultiGetOptions();
   raw_values->resize(keys.size());
   std::vector<rocksdb::Status> statuses(keys.size());
   std::vector<rocksdb::PinnableSlice> pin_values(keys.size());
-  storage_->MultiGet(read_options, metadata_cf_handle_, keys.size(), keys.data(), pin_values.data(), statuses.data());
+  storage_->MultiGet(ctx, read_options, metadata_cf_handle_, keys.size(), keys.data(), pin_values.data(),
+                     statuses.data());
   for (size_t i = 0; i < keys.size(); i++) {
     if (!statuses[i].ok()) continue;
     (*raw_values)[i].assign(pin_values[i].data(), pin_values[i].size());
@@ -59,10 +57,10 @@ std::vector<rocksdb::Status> String::getRawValues(const std::vector<Slice> &keys
   return statuses;
 }
 
-rocksdb::Status String::getRawValue(const std::string &ns_key, std::string *raw_value) {
+rocksdb::Status String::getRawValue(engine::Context &ctx, const std::string &ns_key, std::string *raw_value) {
   raw_value->clear();
 
-  auto s = GetRawMetadata(GetOptions{}, ns_key, raw_value);
+  auto s = GetRawMetadata(ctx, ns_key, raw_value);
   if (!s.ok()) return s;
 
   Metadata metadata(kRedisNone, false);
@@ -70,11 +68,12 @@ rocksdb::Status String::getRawValue(const std::string &ns_key, std::string *raw_
   return ParseMetadata({kRedisString}, &slice, &metadata);
 }
 
-rocksdb::Status String::getValueAndExpire(const std::string &ns_key, std::string *value, uint64_t *expire) {
+rocksdb::Status String::getValueAndExpire(engine::Context &ctx, const std::string &ns_key, std::string *value,
+                                          uint64_t *expire) {
   value->clear();
 
   std::string raw_value;
-  auto s = getRawValue(ns_key, &raw_value);
+  auto s = getRawValue(ctx, ns_key, &raw_value);
   if (!s.ok()) return s;
 
   size_t offset = Metadata::GetOffsetAfterExpire(raw_value[0]);
@@ -89,12 +88,13 @@ rocksdb::Status String::getValueAndExpire(const std::string &ns_key, std::string
   return rocksdb::Status::OK();
 }
 
-rocksdb::Status String::getValue(const std::string &ns_key, std::string *value) {
-  return getValueAndExpire(ns_key, value, nullptr);
+rocksdb::Status String::getValue(engine::Context &ctx, const std::string &ns_key, std::string *value) {
+  return getValueAndExpire(ctx, ns_key, value, nullptr);
 }
 
-std::vector<rocksdb::Status> String::getValues(const std::vector<Slice> &ns_keys, std::vector<std::string> *values) {
-  auto statuses = getRawValues(ns_keys, values);
+std::vector<rocksdb::Status> String::getValues(engine::Context &ctx, const std::vector<Slice> &ns_keys,
+                                               std::vector<std::string> *values) {
+  auto statuses = getRawValues(ctx, ns_keys, values);
   for (size_t i = 0; i < ns_keys.size(); i++) {
     if (!statuses[i].ok()) continue;
     size_t offset = Metadata::GetOffsetAfterExpire((*values)[i][0]);
@@ -103,21 +103,24 @@ std::vector<rocksdb::Status> String::getValues(const std::vector<Slice> &ns_keys
   return statuses;
 }
 
-rocksdb::Status String::updateRawValue(const std::string &ns_key, const std::string &raw_value) {
+rocksdb::Status String::updateRawValue(engine::Context &ctx, const std::string &ns_key, const std::string &raw_value) {
   auto batch = storage_->GetWriteBatchBase();
   WriteBatchLogData log_data(kRedisString);
-  batch->PutLogData(log_data.Encode());
-  batch->Put(metadata_cf_handle_, ns_key, raw_value);
-  return storage_->Write(storage_->DefaultWriteOptions(), batch->GetWriteBatch());
+  auto s = batch->PutLogData(log_data.Encode());
+  if (!s.ok()) return s;
+  s = batch->Put(metadata_cf_handle_, ns_key, raw_value);
+  if (!s.ok()) return s;
+  return storage_->Write(ctx, storage_->DefaultWriteOptions(), batch->GetWriteBatch());
 }
 
-rocksdb::Status String::Append(const std::string &user_key, const std::string &value, uint64_t *new_size) {
+rocksdb::Status String::Append(engine::Context &ctx, const std::string &user_key, const std::string &value,
+                               uint64_t *new_size) {
   *new_size = 0;
   std::string ns_key = AppendNamespacePrefix(user_key);
 
   LockGuard guard(storage_->GetLockManager(), ns_key);
   std::string raw_value;
-  rocksdb::Status s = getRawValue(ns_key, &raw_value);
+  rocksdb::Status s = getRawValue(ctx, ns_key, &raw_value);
   if (!s.ok() && !s.IsNotFound()) return s;
   if (s.IsNotFound()) {
     Metadata metadata(kRedisString, false);
@@ -125,10 +128,11 @@ rocksdb::Status String::Append(const std::string &user_key, const std::string &v
   }
   raw_value.append(value);
   *new_size = raw_value.size() - Metadata::GetOffsetAfterExpire(raw_value[0]);
-  return updateRawValue(ns_key, raw_value);
+  return updateRawValue(ctx, ns_key, raw_value);
 }
 
-std::vector<rocksdb::Status> String::MGet(const std::vector<Slice> &keys, std::vector<std::string> *values) {
+std::vector<rocksdb::Status> String::MGet(engine::Context &ctx, const std::vector<Slice> &keys,
+                                          std::vector<std::string> *values) {
   std::vector<std::string> ns_keys;
   ns_keys.reserve(keys.size());
   for (const auto &key : keys) {
@@ -140,19 +144,20 @@ std::vector<rocksdb::Status> String::MGet(const std::vector<Slice> &keys, std::v
   for (const auto &ns_key : ns_keys) {
     slice_keys.emplace_back(ns_key);
   }
-  return getValues(slice_keys, values);
+  return getValues(ctx, slice_keys, values);
 }
 
-rocksdb::Status String::Get(const std::string &user_key, std::string *value) {
+rocksdb::Status String::Get(engine::Context &ctx, const std::string &user_key, std::string *value) {
   std::string ns_key = AppendNamespacePrefix(user_key);
-  return getValue(ns_key, value);
+  return getValue(ctx, ns_key, value);
 }
 
-rocksdb::Status String::GetEx(const std::string &user_key, std::string *value, std::optional<uint64_t> expire) {
+rocksdb::Status String::GetEx(engine::Context &ctx, const std::string &user_key, std::string *value,
+                              std::optional<uint64_t> expire) {
   std::string ns_key = AppendNamespacePrefix(user_key);
 
   LockGuard guard(storage_->GetLockManager(), ns_key);
-  rocksdb::Status s = getValue(ns_key, value);
+  rocksdb::Status s = getValue(ctx, ns_key, value);
   if (!s.ok()) return s;
 
   std::string raw_data;
@@ -167,35 +172,38 @@ rocksdb::Status String::GetEx(const std::string &user_key, std::string *value, s
   raw_data.append(value->data(), value->size());
   auto batch = storage_->GetWriteBatchBase();
   WriteBatchLogData log_data(kRedisString);
-  batch->PutLogData(log_data.Encode());
-  batch->Put(metadata_cf_handle_, ns_key, raw_data);
-  s = storage_->Write(storage_->DefaultWriteOptions(), batch->GetWriteBatch());
+  s = batch->PutLogData(log_data.Encode());
+  if (!s.ok()) return s;
+  s = batch->Put(metadata_cf_handle_, ns_key, raw_data);
+  if (!s.ok()) return s;
+  s = storage_->Write(ctx, storage_->DefaultWriteOptions(), batch->GetWriteBatch());
   if (!s.ok()) return s;
   return rocksdb::Status::OK();
 }
 
-rocksdb::Status String::GetSet(const std::string &user_key, const std::string &new_value,
+rocksdb::Status String::GetSet(engine::Context &ctx, const std::string &user_key, const std::string &new_value,
                                std::optional<std::string> &old_value) {
-  auto s = Set(user_key, new_value, {/*expire=*/0, StringSetType::NONE, /*get=*/true, /*keep_ttl=*/false}, old_value);
+  auto s =
+      Set(ctx, user_key, new_value, {/*expire=*/0, StringSetType::NONE, /*get=*/true, /*keep_ttl=*/false}, old_value);
   return s;
 }
-rocksdb::Status String::GetDel(const std::string &user_key, std::string *value) {
+rocksdb::Status String::GetDel(engine::Context &ctx, const std::string &user_key, std::string *value) {
   std::string ns_key = AppendNamespacePrefix(user_key);
 
   LockGuard guard(storage_->GetLockManager(), ns_key);
-  rocksdb::Status s = getValue(ns_key, value);
+  rocksdb::Status s = getValue(ctx, ns_key, value);
   if (!s.ok()) return s;
 
-  return storage_->Delete(storage_->DefaultWriteOptions(), metadata_cf_handle_, ns_key);
+  return storage_->Delete(ctx, storage_->DefaultWriteOptions(), metadata_cf_handle_, ns_key);
 }
 
-rocksdb::Status String::Set(const std::string &user_key, const std::string &value) {
+rocksdb::Status String::Set(engine::Context &ctx, const std::string &user_key, const std::string &value) {
   std::vector<StringPair> pairs{StringPair{user_key, value}};
-  return MSet(pairs, /*expire=*/0, /*lock=*/true);
+  return MSet(ctx, pairs, /*expire=*/0, /*lock=*/true);
 }
 
-rocksdb::Status String::Set(const std::string &user_key, const std::string &value, StringSetArgs args,
-                            std::optional<std::string> &ret) {
+rocksdb::Status String::Set(engine::Context &ctx, const std::string &user_key, const std::string &value,
+                            StringSetArgs args, std::optional<std::string> &ret) {
   uint64_t expire = 0;
   std::string ns_key = AppendNamespacePrefix(user_key);
 
@@ -204,7 +212,7 @@ rocksdb::Status String::Set(const std::string &user_key, const std::string &valu
   if (need_old_value) {
     std::string old_value;
     uint64_t old_expire = 0;
-    auto s = getValueAndExpire(ns_key, &old_value, &old_expire);
+    auto s = getValueAndExpire(ctx, ns_key, &old_value, &old_expire);
     if (!s.ok() && !s.IsNotFound() && !s.IsInvalidArgument()) return s;
     // GET option
     if (args.get) {
@@ -252,35 +260,38 @@ rocksdb::Status String::Set(const std::string &user_key, const std::string &valu
   metadata.expire = expire;
   metadata.Encode(&new_raw_value);
   new_raw_value.append(value);
-  return updateRawValue(ns_key, new_raw_value);
+  return updateRawValue(ctx, ns_key, new_raw_value);
 }
 
-rocksdb::Status String::SetEX(const std::string &user_key, const std::string &value, uint64_t expire_ms) {
+rocksdb::Status String::SetEX(engine::Context &ctx, const std::string &user_key, const std::string &value,
+                              uint64_t expire_ms) {
   std::optional<std::string> ret;
-  return Set(user_key, value, {expire_ms, StringSetType::NONE, /*get=*/false, /*keep_ttl=*/false}, ret);
+  return Set(ctx, user_key, value, {expire_ms, StringSetType::NONE, /*get=*/false, /*keep_ttl=*/false}, ret);
 }
 
-rocksdb::Status String::SetNX(const std::string &user_key, const std::string &value, uint64_t expire_ms, bool *flag) {
+rocksdb::Status String::SetNX(engine::Context &ctx, const std::string &user_key, const std::string &value,
+                              uint64_t expire_ms, bool *flag) {
   std::optional<std::string> ret;
-  auto s = Set(user_key, value, {expire_ms, StringSetType::NX, /*get=*/false, /*keep_ttl=*/false}, ret);
+  auto s = Set(ctx, user_key, value, {expire_ms, StringSetType::NX, /*get=*/false, /*keep_ttl=*/false}, ret);
   *flag = ret.has_value();
   return s;
 }
 
-rocksdb::Status String::SetXX(const std::string &user_key, const std::string &value, uint64_t expire_ms, bool *flag) {
+rocksdb::Status String::SetXX(engine::Context &ctx, const std::string &user_key, const std::string &value,
+                              uint64_t expire_ms, bool *flag) {
   std::optional<std::string> ret;
-  auto s = Set(user_key, value, {expire_ms, StringSetType::XX, /*get=*/false, /*keep_ttl=*/false}, ret);
+  auto s = Set(ctx, user_key, value, {expire_ms, StringSetType::XX, /*get=*/false, /*keep_ttl=*/false}, ret);
   *flag = ret.has_value();
   return s;
 }
 
-rocksdb::Status String::SetRange(const std::string &user_key, size_t offset, const std::string &value,
-                                 uint64_t *new_size) {
+rocksdb::Status String::SetRange(engine::Context &ctx, const std::string &user_key, size_t offset,
+                                 const std::string &value, uint64_t *new_size) {
   std::string ns_key = AppendNamespacePrefix(user_key);
 
   LockGuard guard(storage_->GetLockManager(), ns_key);
   std::string raw_value;
-  rocksdb::Status s = getRawValue(ns_key, &raw_value);
+  rocksdb::Status s = getRawValue(ctx, ns_key, &raw_value);
   if (!s.ok() && !s.IsNotFound()) return s;
 
   if (s.IsNotFound()) {
@@ -311,15 +322,16 @@ rocksdb::Status String::SetRange(const std::string &user_key, size_t offset, con
     }
   }
   *new_size = raw_value.size() - header_offset;
-  return updateRawValue(ns_key, raw_value);
+  return updateRawValue(ctx, ns_key, raw_value);
 }
 
-rocksdb::Status String::IncrBy(const std::string &user_key, int64_t increment, int64_t *new_value) {
+rocksdb::Status String::IncrBy(engine::Context &ctx, const std::string &user_key, int64_t increment,
+                               int64_t *new_value) {
   std::string ns_key = AppendNamespacePrefix(user_key);
 
   LockGuard guard(storage_->GetLockManager(), ns_key);
   std::string raw_value;
-  rocksdb::Status s = getRawValue(ns_key, &raw_value);
+  rocksdb::Status s = getRawValue(ctx, ns_key, &raw_value);
   if (!s.ok() && !s.IsNotFound()) return s;
   if (s.IsNotFound()) {
     Metadata metadata(kRedisString, false);
@@ -348,14 +360,15 @@ rocksdb::Status String::IncrBy(const std::string &user_key, int64_t increment, i
 
   raw_value = raw_value.substr(0, offset);
   raw_value.append(std::to_string(n));
-  return updateRawValue(ns_key, raw_value);
+  return updateRawValue(ctx, ns_key, raw_value);
 }
 
-rocksdb::Status String::IncrByFloat(const std::string &user_key, double increment, double *new_value) {
+rocksdb::Status String::IncrByFloat(engine::Context &ctx, const std::string &user_key, double increment,
+                                    double *new_value) {
   std::string ns_key = AppendNamespacePrefix(user_key);
   LockGuard guard(storage_->GetLockManager(), ns_key);
   std::string raw_value;
-  rocksdb::Status s = getRawValue(ns_key, &raw_value);
+  rocksdb::Status s = getRawValue(ctx, ns_key, &raw_value);
   if (!s.ok() && !s.IsNotFound()) return s;
 
   if (s.IsNotFound()) {
@@ -381,10 +394,11 @@ rocksdb::Status String::IncrByFloat(const std::string &user_key, double incremen
 
   raw_value = raw_value.substr(0, offset);
   raw_value.append(std::to_string(n));
-  return updateRawValue(ns_key, raw_value);
+  return updateRawValue(ctx, ns_key, raw_value);
 }
 
-rocksdb::Status String::MSet(const std::vector<StringPair> &pairs, uint64_t expire_ms, bool lock) {
+rocksdb::Status String::MSet(engine::Context &ctx, const std::vector<StringPair> &pairs, uint64_t expire_ms,
+                             bool lock) {
   // Data race, key string maybe overwrite by other key while didn't lock the keys here,
   // to improve the set performance
   std::optional<MultiLockGuard> guard;
@@ -400,7 +414,8 @@ rocksdb::Status String::MSet(const std::vector<StringPair> &pairs, uint64_t expi
 
   auto batch = storage_->GetWriteBatchBase();
   WriteBatchLogData log_data(kRedisString);
-  batch->PutLogData(log_data.Encode());
+  auto s = batch->PutLogData(log_data.Encode());
+  if (!s.ok()) return s;
   for (const auto &pair : pairs) {
     std::string bytes;
     Metadata metadata(kRedisString, false);
@@ -408,12 +423,14 @@ rocksdb::Status String::MSet(const std::vector<StringPair> &pairs, uint64_t expi
     metadata.Encode(&bytes);
     bytes.append(pair.value.data(), pair.value.size());
     std::string ns_key = AppendNamespacePrefix(pair.key);
-    batch->Put(metadata_cf_handle_, ns_key, bytes);
+    s = batch->Put(metadata_cf_handle_, ns_key, bytes);
+    if (!s.ok()) return s;
   }
-  return storage_->Write(storage_->DefaultWriteOptions(), batch->GetWriteBatch());
+  return storage_->Write(ctx, storage_->DefaultWriteOptions(), batch->GetWriteBatch());
 }
 
-rocksdb::Status String::MSetNX(const std::vector<StringPair> &pairs, uint64_t expire_ms, bool *flag) {
+rocksdb::Status String::MSetNX(engine::Context &ctx, const std::vector<StringPair> &pairs, uint64_t expire_ms,
+                               bool *flag) {
   *flag = false;
 
   int exists = 0;
@@ -431,11 +448,11 @@ rocksdb::Status String::MSetNX(const std::vector<StringPair> &pairs, uint64_t ex
   // Lock these keys before doing anything.
   MultiLockGuard guard(storage_->GetLockManager(), lock_keys);
 
-  if (Exists(keys, &exists).ok() && exists > 0) {
+  if (Exists(ctx, keys, &exists).ok() && exists > 0) {
     return rocksdb::Status::OK();
   }
 
-  rocksdb::Status s = MSet(pairs, /*expire_ms=*/expire_ms, /*lock=*/false);
+  rocksdb::Status s = MSet(ctx, pairs, /*expire_ms=*/expire_ms, /*lock=*/false);
   if (!s.ok()) return s;
 
   *flag = true;
@@ -447,15 +464,15 @@ rocksdb::Status String::MSetNX(const std::vector<StringPair> &pairs, uint64_t ex
 //  1 if the operation is successful
 //  -1 if the user_key does not exist
 //  0 if the operation fails
-rocksdb::Status String::CAS(const std::string &user_key, const std::string &old_value, const std::string &new_value,
-                            uint64_t expire, int *flag) {
+rocksdb::Status String::CAS(engine::Context &ctx, const std::string &user_key, const std::string &old_value,
+                            const std::string &new_value, uint64_t expire, int *flag) {
   *flag = 0;
 
   std::string current_value;
   std::string ns_key = AppendNamespacePrefix(user_key);
 
   LockGuard guard(storage_->GetLockManager(), ns_key);
-  rocksdb::Status s = getValue(ns_key, &current_value);
+  rocksdb::Status s = getValue(ctx, ns_key, &current_value);
 
   if (!s.ok() && !s.IsNotFound()) {
     return s;
@@ -472,7 +489,7 @@ rocksdb::Status String::CAS(const std::string &user_key, const std::string &old_
     metadata.expire = expire;
     metadata.Encode(&raw_value);
     raw_value.append(new_value);
-    auto write_status = updateRawValue(ns_key, raw_value);
+    auto write_status = updateRawValue(ctx, ns_key, raw_value);
     if (!write_status.ok()) {
       return write_status;
     }
@@ -484,14 +501,14 @@ rocksdb::Status String::CAS(const std::string &user_key, const std::string &old_
 
 // Delete a specified user_key if the current value of the user_key matches a specified value.
 // For ret, same as CAS.
-rocksdb::Status String::CAD(const std::string &user_key, const std::string &value, int *flag) {
+rocksdb::Status String::CAD(engine::Context &ctx, const std::string &user_key, const std::string &value, int *flag) {
   *flag = 0;
 
   std::string current_value;
   std::string ns_key = AppendNamespacePrefix(user_key);
 
   LockGuard guard(storage_->GetLockManager(), ns_key);
-  rocksdb::Status s = getValue(ns_key, &current_value);
+  rocksdb::Status s = getValue(ctx, ns_key, &current_value);
 
   if (!s.ok() && !s.IsNotFound()) {
     return s;
@@ -504,7 +521,7 @@ rocksdb::Status String::CAD(const std::string &user_key, const std::string &valu
 
   if (value == current_value) {
     auto delete_status =
-        storage_->Delete(storage_->DefaultWriteOptions(), storage_->GetCFHandle(ColumnFamilyID::Metadata), ns_key);
+        storage_->Delete(ctx, storage_->DefaultWriteOptions(), storage_->GetCFHandle(ColumnFamilyID::Metadata), ns_key);
     if (!delete_status.ok()) {
       return delete_status;
     }
@@ -514,8 +531,8 @@ rocksdb::Status String::CAD(const std::string &user_key, const std::string &valu
   return rocksdb::Status::OK();
 }
 
-rocksdb::Status String::LCS(const std::string &user_key1, const std::string &user_key2, StringLCSArgs args,
-                            StringLCSResult *rst) {
+rocksdb::Status String::LCS(engine::Context &ctx, const std::string &user_key1, const std::string &user_key2,
+                            StringLCSArgs args, StringLCSResult *rst) {
   if (args.type == StringLCSType::LEN) {
     *rst = static_cast<uint32_t>(0);
   } else if (args.type == StringLCSType::IDX) {
@@ -528,8 +545,8 @@ rocksdb::Status String::LCS(const std::string &user_key1, const std::string &use
   std::string b;
   std::string ns_key1 = AppendNamespacePrefix(user_key1);
   std::string ns_key2 = AppendNamespacePrefix(user_key2);
-  auto s1 = getValue(ns_key1, &a);
-  auto s2 = getValue(ns_key2, &b);
+  auto s1 = getValue(ctx, ns_key1, &a);
+  auto s2 = getValue(ctx, ns_key2, &b);
 
   if (!s1.ok() && !s1.IsNotFound()) {
     return s1;
@@ -553,7 +570,7 @@ rocksdb::Status String::LCS(const std::string &user_key1, const std::string &use
   // Allocate the LCS table.
   uint64_t dp_size = (alen + 1) * (blen + 1);
   uint64_t bulk_size = dp_size * sizeof(uint32_t);
-  if (bulk_size > PROTO_BULK_MAX_SIZE || bulk_size / dp_size != sizeof(uint32_t)) {
+  if (bulk_size > storage_->GetConfig()->proto_max_bulk_len || bulk_size / dp_size != sizeof(uint32_t)) {
     return rocksdb::Status::Aborted("Insufficient memory, transient memory for LCS exceeds proto-max-bulk-len");
   }
   std::vector<uint32_t> dp(dp_size, 0);
@@ -597,10 +614,10 @@ rocksdb::Status String::LCS(const std::string &user_key1, const std::string &use
 
   uint32_t i = alen;
   uint32_t j = blen;
-  uint32_t arange_start = alen;  // alen signals that values are not set.
-  uint32_t arange_end = 0;
-  uint32_t brange_start = 0;
-  uint32_t brange_end = 0;
+  uint32_t a_range_start = alen;  // alen signals that values are not set.
+  uint32_t a_range_end = 0;
+  uint32_t b_range_start = 0;
+  uint32_t b_range_end = 0;
   while (i > 0 && j > 0) {
     bool emit_range = false;
     if (a[i - 1] == b[j - 1]) {
@@ -611,24 +628,24 @@ rocksdb::Status String::LCS(const std::string &user_key1, const std::string &use
       }
 
       // Track the current range.
-      if (arange_start == alen) {
-        arange_start = i - 1;
-        arange_end = i - 1;
-        brange_start = j - 1;
-        brange_end = j - 1;
+      if (a_range_start == alen) {
+        a_range_start = i - 1;
+        a_range_end = i - 1;
+        b_range_start = j - 1;
+        b_range_end = j - 1;
       }
       // Let's see if we can extend the range backward since
       // it is contiguous.
-      else if (arange_start == i && brange_start == j) {
-        arange_start--;
-        brange_start--;
+      else if (a_range_start == i && b_range_start == j) {
+        a_range_start--;
+        b_range_start--;
       } else {
         emit_range = true;
       }
 
       // Emit the range if we matched with the first byte of
       // one of the two strings. We'll exit the loop ASAP.
-      if (arange_start == 0 || brange_start == 0) {
+      if (a_range_start == 0 || b_range_start == 0) {
         emit_range = true;
       }
       idx--;
@@ -643,23 +660,23 @@ rocksdb::Status String::LCS(const std::string &user_key1, const std::string &use
         i--;
       else
         j--;
-      if (arange_start != alen) emit_range = true;
+      if (a_range_start != alen) emit_range = true;
     }
 
     // Emit the current range if needed.
     if (emit_range) {
       if (auto result = std::get_if<StringLCSIdxResult>(rst)) {
-        uint32_t match_len = arange_end - arange_start + 1;
+        uint32_t match_len = a_range_end - a_range_start + 1;
 
         // Always emit the range when the `min_match_len` is not set.
         if (args.min_match_len == 0 || match_len >= args.min_match_len) {
-          result->matches.emplace_back(StringLCSRange{arange_start, arange_end},
-                                       StringLCSRange{brange_start, brange_end}, match_len);
+          result->matches.emplace_back(StringLCSRange{a_range_start, a_range_end},
+                                       StringLCSRange{b_range_start, b_range_end}, match_len);
         }
       }
 
       // Restart at the next match.
-      arange_start = alen;
+      a_range_start = alen;
     }
   }
 
