@@ -56,14 +56,14 @@ class RDBTest : public TestBase {
     ASSERT_TRUE(s.IsOK());
 
     RDB rdb(storage_.get(), ns_, std::move(stream_ptr));
-    s = rdb.LoadRdb(0);
+    s = rdb.LoadRdb(*ctx_, 0);
     ASSERT_TRUE(s.IsOK());
   }
 
   void stringCheck(const std::string &key, const std::string &expect) {
     redis::String string_db(storage_.get(), ns_);
     std::string value;
-    auto s = string_db.Get(key, &value);
+    auto s = string_db.Get(*ctx_, key, &value);
     ASSERT_TRUE(s.ok());
     ASSERT_TRUE(expect == value);
   }
@@ -71,8 +71,7 @@ class RDBTest : public TestBase {
   void setCheck(const std::string &key, const std::vector<std::string> &expect) {
     redis::Set set_db(storage_.get(), ns_);
     std::vector<std::string> members;
-    auto s = set_db.Members(key, &members);
-
+    auto s = set_db.Members(*ctx_, key, &members);
     ASSERT_TRUE(s.ok());
     ASSERT_TRUE(expect == members);
   }
@@ -80,7 +79,7 @@ class RDBTest : public TestBase {
   void hashCheck(const std::string &key, const std::map<std::string, std::string> &expect) {
     redis::Hash hash_db(storage_.get(), ns_);
     std::vector<FieldValue> field_values;
-    auto s = hash_db.GetAll(key, &field_values);
+    auto s = hash_db.GetAll(*ctx_, key, &field_values);
     ASSERT_TRUE(s.ok());
 
     // size check
@@ -97,7 +96,7 @@ class RDBTest : public TestBase {
   void listCheck(const std::string &key, const std::vector<std::string> &expect) {
     redis::List list_db(storage_.get(), ns_);
     std::vector<std::string> values;
-    auto s = list_db.Range(key, 0, -1, &values);
+    auto s = list_db.Range(*ctx_, key, 0, -1, &values);
     ASSERT_TRUE(s.ok());
     ASSERT_TRUE(expect == values);
   }
@@ -107,7 +106,7 @@ class RDBTest : public TestBase {
     std::vector<MemberScore> member_scores;
 
     RangeRankSpec spec;
-    auto s = zset_db.RangeByRank(key, spec, &member_scores, nullptr);
+    auto s = zset_db.RangeByRank(*ctx_, key, spec, &member_scores, nullptr);
     ASSERT_TRUE(s.ok());
     ASSERT_TRUE(expect.size() == member_scores.size());
     for (size_t i = 0; i < expect.size(); ++i) {
@@ -118,12 +117,12 @@ class RDBTest : public TestBase {
 
   rocksdb::Status keyExist(const std::string &key) {
     redis::Database redis(storage_.get(), ns_);
-    return redis.KeyExist(key);
+    return redis.KeyExist(*ctx_, key);
   }
 
   void flushDB() {
     redis::Database redis(storage_.get(), ns_);
-    auto s = redis.FlushDB();
+    auto s = redis.FlushDB(*ctx_);
     ASSERT_TRUE(s.ok());
   }
 
