@@ -38,12 +38,12 @@ class RedisBitmapTest : public TestFixture, public ::testing::TestWithParam<bool
     key_ = "test_bitmap_key";
     if (bool use_bitmap = GetParam(); !use_bitmap) {
       // Set an empty string.
-      string_->Set(key_, "");
+      string_->Set(*ctx_, key_, "");
     }
   }
   void TearDown() override {
-    [[maybe_unused]] auto s = bitmap_->Del(key_);
-    s = string_->Del(key_);
+    [[maybe_unused]] auto s = bitmap_->Del(*ctx_, key_);
+    s = string_->Del(*ctx_, key_);
   }
 
   std::unique_ptr<redis::Bitmap> bitmap_;
@@ -56,84 +56,84 @@ TEST_P(RedisBitmapTest, GetAndSetBit) {
   uint32_t offsets[] = {0, 123, 1024 * 8, 1024 * 8 + 1, 3 * 1024 * 8, 3 * 1024 * 8 + 1};
   for (const auto &offset : offsets) {
     bool bit = false;
-    bitmap_->GetBit(key_, offset, &bit);
+    bitmap_->GetBit(*ctx_, key_, offset, &bit);
     EXPECT_FALSE(bit);
-    bitmap_->SetBit(key_, offset, true, &bit);
-    bitmap_->GetBit(key_, offset, &bit);
+    bitmap_->SetBit(*ctx_, key_, offset, true, &bit);
+    bitmap_->GetBit(*ctx_, key_, offset, &bit);
     EXPECT_TRUE(bit);
   }
-  auto s = bitmap_->Del(key_);
+  auto s = bitmap_->Del(*ctx_, key_);
 }
 
 TEST_P(RedisBitmapTest, BitCount) {
   uint32_t offsets[] = {0, 123, 1024 * 8, 1024 * 8 + 1, 3 * 1024 * 8, 3 * 1024 * 8 + 1};
   for (const auto &offset : offsets) {
     bool bit = false;
-    bitmap_->SetBit(key_, offset, true, &bit);
+    bitmap_->SetBit(*ctx_, key_, offset, true, &bit);
   }
   uint32_t cnt = 0;
-  bitmap_->BitCount(key_, 0, 4 * 1024, false, &cnt);
+  bitmap_->BitCount(*ctx_, key_, 0, 4 * 1024, false, &cnt);
   EXPECT_EQ(cnt, 6);
-  bitmap_->BitCount(key_, 0, -1, false, &cnt);
+  bitmap_->BitCount(*ctx_, key_, 0, -1, false, &cnt);
   EXPECT_EQ(cnt, 6);
-  auto s = bitmap_->Del(key_);
+  auto s = bitmap_->Del(*ctx_, key_);
 }
 
 TEST_P(RedisBitmapTest, BitCountNegative) {
   {
     bool bit = false;
-    bitmap_->SetBit(key_, 0, true, &bit);
+    bitmap_->SetBit(*ctx_, key_, 0, true, &bit);
     EXPECT_FALSE(bit);
   }
   uint32_t cnt = 0;
-  bitmap_->BitCount(key_, 0, 4 * 1024, false, &cnt);
+  bitmap_->BitCount(*ctx_, key_, 0, 4 * 1024, false, &cnt);
   EXPECT_EQ(cnt, 1);
-  bitmap_->BitCount(key_, 0, 0, false, &cnt);
+  bitmap_->BitCount(*ctx_, key_, 0, 0, false, &cnt);
   EXPECT_EQ(cnt, 1);
-  bitmap_->BitCount(key_, 0, -1, false, &cnt);
+  bitmap_->BitCount(*ctx_, key_, 0, -1, false, &cnt);
   EXPECT_EQ(cnt, 1);
-  bitmap_->BitCount(key_, -1, -1, false, &cnt);
+  bitmap_->BitCount(*ctx_, key_, -1, -1, false, &cnt);
   EXPECT_EQ(cnt, 1);
-  bitmap_->BitCount(key_, 1, 1, false, &cnt);
+  bitmap_->BitCount(*ctx_, key_, 1, 1, false, &cnt);
   EXPECT_EQ(cnt, 0);
-  bitmap_->BitCount(key_, -10000, -10000, false, &cnt);
+  bitmap_->BitCount(*ctx_, key_, -10000, -10000, false, &cnt);
   EXPECT_EQ(cnt, 1);
 
   {
     bool bit = false;
-    bitmap_->SetBit(key_, 5, true, &bit);
+    bitmap_->SetBit(*ctx_, key_, 5, true, &bit);
     EXPECT_FALSE(bit);
   }
-  bitmap_->BitCount(key_, -10000, -10000, false, &cnt);
+  bitmap_->BitCount(*ctx_, key_, -10000, -10000, false, &cnt);
   EXPECT_EQ(cnt, 2);
 
   {
     bool bit = false;
-    bitmap_->SetBit(key_, 8 * 1024 - 1, true, &bit);
+    bitmap_->SetBit(*ctx_, key_, 8 * 1024 - 1, true, &bit);
     EXPECT_FALSE(bit);
-    bitmap_->SetBit(key_, 8 * 1024, true, &bit);
+    bitmap_->SetBit(*ctx_, key_, 8 * 1024, true, &bit);
     EXPECT_FALSE(bit);
   }
 
-  bitmap_->BitCount(key_, 0, 1024, false, &cnt);
+  bitmap_->BitCount(*ctx_, key_, 0, 1024, false, &cnt);
   EXPECT_EQ(cnt, 4);
 
-  bitmap_->BitCount(key_, 0, 1023, false, &cnt);
+  bitmap_->BitCount(*ctx_, key_, 0, 1023, false, &cnt);
   EXPECT_EQ(cnt, 3);
 
-  auto s = bitmap_->Del(key_);
+  auto s = bitmap_->Del(*ctx_, key_);
 }
 
 TEST_P(RedisBitmapTest, BitCountBITOption) {
   std::set<uint32_t> offsets = {0, 100, 1024 * 8, 1024 * 8 + 1, 3 * 1024 * 8, 3 * 1024 * 8 + 1};
   for (const auto &offset : offsets) {
     bool bit = false;
-    bitmap_->SetBit(key_, offset, true, &bit);
+    bitmap_->SetBit(*ctx_, key_, offset, true, &bit);
   }
 
   for (uint32_t bit_offset = 0; bit_offset <= 3 * 1024 * 8 + 10; ++bit_offset) {
     uint32_t cnt = 0;
-    EXPECT_TRUE(bitmap_->BitCount(key_, bit_offset, bit_offset, true, &cnt).ok());
+    EXPECT_TRUE(bitmap_->BitCount(*ctx_, key_, bit_offset, bit_offset, true, &cnt).ok());
     if (offsets.count(bit_offset) > 0) {
       ASSERT_EQ(1, cnt) << "bit_offset: " << bit_offset;
     } else {
@@ -142,25 +142,25 @@ TEST_P(RedisBitmapTest, BitCountBITOption) {
   }
 
   uint32_t cnt = 0;
-  bitmap_->BitCount(key_, 0, 4 * 1024 * 8, true, &cnt);
+  bitmap_->BitCount(*ctx_, key_, 0, 4 * 1024 * 8, true, &cnt);
   EXPECT_EQ(cnt, 6);
-  bitmap_->BitCount(key_, 0, -1, true, &cnt);
+  bitmap_->BitCount(*ctx_, key_, 0, -1, true, &cnt);
   EXPECT_EQ(cnt, 6);
-  bitmap_->BitCount(key_, 0, 3 * 1024 * 8 + 1, true, &cnt);
+  bitmap_->BitCount(*ctx_, key_, 0, 3 * 1024 * 8 + 1, true, &cnt);
   EXPECT_EQ(cnt, 6);
-  bitmap_->BitCount(key_, 1, 3 * 1024 * 8 + 1, true, &cnt);
+  bitmap_->BitCount(*ctx_, key_, 1, 3 * 1024 * 8 + 1, true, &cnt);
   EXPECT_EQ(cnt, 5);
-  bitmap_->BitCount(key_, 0, 0, true, &cnt);
+  bitmap_->BitCount(*ctx_, key_, 0, 0, true, &cnt);
   EXPECT_EQ(cnt, 1);
-  bitmap_->BitCount(key_, 0, 100, true, &cnt);
+  bitmap_->BitCount(*ctx_, key_, 0, 100, true, &cnt);
   EXPECT_EQ(cnt, 2);
-  bitmap_->BitCount(key_, 100, 1024 * 8, true, &cnt);
+  bitmap_->BitCount(*ctx_, key_, 100, 1024 * 8, true, &cnt);
   EXPECT_EQ(cnt, 2);
-  bitmap_->BitCount(key_, 100, 3 * 1024 * 8, true, &cnt);
+  bitmap_->BitCount(*ctx_, key_, 100, 3 * 1024 * 8, true, &cnt);
   EXPECT_EQ(cnt, 4);
-  bitmap_->BitCount(key_, -1, -1, true, &cnt);
+  bitmap_->BitCount(*ctx_, key_, -1, -1, true, &cnt);
   EXPECT_EQ(cnt, 0);  // NOTICE: the min storage unit is byte, the result is the same as Redis.
-  auto s = bitmap_->Del(key_);
+  auto s = bitmap_->Del(*ctx_, key_);
 }
 
 TEST_P(RedisBitmapTest, BitPosClearBit) {
@@ -179,58 +179,60 @@ TEST_P(RedisBitmapTest, BitPosClearBit) {
     ///
     /// String will set a empty string value when initializing, so, when first
     /// querying, it should return -1.
-    bitmap_->BitPos(key_, false, 0, -1, /*stop_given=*/false, &pos, /*bit_index=*/false);
+    bitmap_->BitPos(*ctx_, key_, false, 0, -1, /*stop_given=*/false, &pos, /*bit_index=*/false);
     if (i == 0 && !use_bitmap) {
       EXPECT_EQ(pos, -1);
     } else {
       EXPECT_EQ(pos, i);
     }
 
-    bitmap_->SetBit(key_, i, true, &old_bit);
+    bitmap_->SetBit(*ctx_, key_, i, true, &old_bit);
     EXPECT_FALSE(old_bit);
+    ctx_->RefreshLatestSnapshot();
   }
-  auto s = bitmap_->Del(key_);
+  auto s = bitmap_->Del(*ctx_, key_);
 }
 
 TEST_P(RedisBitmapTest, BitPosSetBit) {
   uint32_t offsets[] = {0, 123, 1024 * 8, 1024 * 8 + 16, 3 * 1024 * 8, 3 * 1024 * 8 + 16};
   for (const auto &offset : offsets) {
     bool bit = false;
-    bitmap_->SetBit(key_, offset, true, &bit);
+    bitmap_->SetBit(*ctx_, key_, offset, true, &bit);
   }
   int64_t pos = 0;
   int start_indexes[] = {0, 1, 124, 1025, 1027, 3 * 1024 + 1};
   for (size_t i = 0; i < sizeof(start_indexes) / sizeof(start_indexes[0]); i++) {
-    bitmap_->BitPos(key_, true, start_indexes[i], -1, true, &pos, /*bit_index=*/false);
+    bitmap_->BitPos(*ctx_, key_, true, start_indexes[i], -1, true, &pos, /*bit_index=*/false);
     EXPECT_EQ(pos, offsets[i]);
+    ctx_->RefreshLatestSnapshot();
   }
-  auto s = bitmap_->Del(key_);
+  auto s = bitmap_->Del(*ctx_, key_);
 }
 
 TEST_P(RedisBitmapTest, BitPosNegative) {
   {
     bool bit = false;
-    bitmap_->SetBit(key_, 8 * 1024 - 1, true, &bit);
+    bitmap_->SetBit(*ctx_, key_, 8 * 1024 - 1, true, &bit);
     EXPECT_FALSE(bit);
   }
   int64_t pos = 0;
   // First bit is negative
-  bitmap_->BitPos(key_, false, 0, -1, true, &pos, /*bit_index=*/false);
+  bitmap_->BitPos(*ctx_, key_, false, 0, -1, true, &pos, /*bit_index=*/false);
   EXPECT_EQ(0, pos);
   // 8 * 1024 - 1 bit is positive
-  bitmap_->BitPos(key_, true, 0, -1, true, &pos, /*bit_index=*/false);
+  bitmap_->BitPos(*ctx_, key_, true, 0, -1, true, &pos, /*bit_index=*/false);
   EXPECT_EQ(8 * 1024 - 1, pos);
   // First bit in 1023 byte is negative
-  bitmap_->BitPos(key_, false, -1, -1, true, &pos, /*bit_index=*/false);
+  bitmap_->BitPos(*ctx_, key_, false, -1, -1, true, &pos, /*bit_index=*/false);
   EXPECT_EQ(8 * 1023, pos);
   // Last Bit in 1023 byte is positive
-  bitmap_->BitPos(key_, true, -1, -1, true, &pos, /*bit_index=*/false);
+  bitmap_->BitPos(*ctx_, key_, true, -1, -1, true, &pos, /*bit_index=*/false);
   EXPECT_EQ(8 * 1024 - 1, pos);
   // Large negative number will be normalized.
-  bitmap_->BitPos(key_, false, -10000, -10000, true, &pos, /*bit_index=*/false);
+  bitmap_->BitPos(*ctx_, key_, false, -10000, -10000, true, &pos, /*bit_index=*/false);
   EXPECT_EQ(0, pos);
 
-  auto s = bitmap_->Del(key_);
+  auto s = bitmap_->Del(*ctx_, key_);
 }
 
 // When `stop_given` is true, even searching for 0,
@@ -238,27 +240,27 @@ TEST_P(RedisBitmapTest, BitPosNegative) {
 TEST_P(RedisBitmapTest, BitPosStopGiven) {
   for (int i = 0; i < 8; ++i) {
     bool bit = true;
-    bitmap_->SetBit(key_, i, true, &bit);
+    bitmap_->SetBit(*ctx_, key_, i, true, &bit);
     EXPECT_FALSE(bit);
   }
   int64_t pos = 0;
-  bitmap_->BitPos(key_, false, 0, 0, /*stop_given=*/true, &pos, /*bit_index=*/false);
+  bitmap_->BitPos(*ctx_, key_, false, 0, 0, /*stop_given=*/true, &pos, /*bit_index=*/false);
   EXPECT_EQ(-1, pos);
-  bitmap_->BitPos(key_, false, 0, 0, /*stop_given=*/false, &pos, /*bit_index=*/false);
+  bitmap_->BitPos(*ctx_, key_, false, 0, 0, /*stop_given=*/false, &pos, /*bit_index=*/false);
   EXPECT_EQ(8, pos);
 
   // Set a bit at 8 not affect that
   {
     bool bit = true;
-    bitmap_->SetBit(key_, 8, true, &bit);
+    bitmap_->SetBit(*ctx_, key_, 8, true, &bit);
     EXPECT_FALSE(bit);
   }
-  bitmap_->BitPos(key_, false, 0, 0, /*stop_given=*/true, &pos, /*bit_index=*/false);
+  bitmap_->BitPos(*ctx_, key_, false, 0, 0, /*stop_given=*/true, &pos, /*bit_index=*/false);
   EXPECT_EQ(-1, pos);
-  bitmap_->BitPos(key_, false, 0, 1, /*stop_given=*/false, &pos, /*bit_index=*/false);
+  bitmap_->BitPos(*ctx_, key_, false, 0, 1, /*stop_given=*/false, &pos, /*bit_index=*/false);
   EXPECT_EQ(9, pos);
 
-  auto s = bitmap_->Del(key_);
+  auto s = bitmap_->Del(*ctx_, key_);
 }
 
 TEST_P(RedisBitmapTest, BitfieldGetSetTest) {
@@ -272,7 +274,7 @@ TEST_P(RedisBitmapTest, BitfieldGetSetTest) {
   op.offset = 114514;
   op.value = magic;
 
-  EXPECT_TRUE(bitmap_->Bitfield(key_, {op}, &rets).ok());
+  EXPECT_TRUE(bitmap_->Bitfield(*ctx_, key_, {op}, &rets).ok());
   EXPECT_EQ(1, rets.size());
   EXPECT_EQ(0, rets[0].value());
   rets.clear();
@@ -282,13 +284,13 @@ TEST_P(RedisBitmapTest, BitfieldGetSetTest) {
 
   // bitfield is stored in big-endian.
   for (int i = 31; i != -1; --i) {
-    EXPECT_TRUE(bitmap_->Bitfield(key_, {op}, &rets).ok());
+    EXPECT_TRUE(bitmap_->Bitfield(*ctx_, key_, {op}, &rets).ok());
     EXPECT_EQ((magic >> i) & 1, rets[0].value());
     rets.clear();
     op.offset++;
   }
 
-  auto s = bitmap_->Del(key_);
+  auto s = bitmap_->Del(*ctx_, key_);
 }
 
 TEST_P(RedisBitmapTest, UnsignedBitfieldTest) {
@@ -304,20 +306,20 @@ TEST_P(RedisBitmapTest, UnsignedBitfieldTest) {
   op.offset = 8189;  // the two bitmap segments divide the bitfield
   op.value = 0;
 
-  bitmap_->Bitfield(key_, {op}, &rets);
+  bitmap_->Bitfield(*ctx_, key_, {op}, &rets);
   EXPECT_EQ(1, rets.size());
   EXPECT_EQ(0, rets[0].value());
   rets.clear();
 
   for (uint64_t i = 1; i <= max; ++i) {
     op.value = int64_t(i);
-    bitmap_->Bitfield(key_, {op}, &rets);
+    bitmap_->Bitfield(*ctx_, key_, {op}, &rets);
     EXPECT_EQ(1, rets.size());
     EXPECT_EQ(i - 1, rets[0].value());
     rets.clear();
   }
 
-  auto s = bitmap_->Del(key_);
+  auto s = bitmap_->Del(*ctx_, key_);
 }
 
 TEST_P(RedisBitmapTest, SignedBitfieldTest) {
@@ -333,19 +335,20 @@ TEST_P(RedisBitmapTest, SignedBitfieldTest) {
   op.offset = 8189;  // the two bitmap segments divide the bitfield
   op.value = min;
 
-  bitmap_->Bitfield(key_, {op}, &rets);
+  bitmap_->Bitfield(*ctx_, key_, {op}, &rets);
   EXPECT_EQ(1, rets.size());
   EXPECT_EQ(0, rets[0].value());
   rets.clear();
 
   for (int64_t i = min + 1; i <= max; ++i) {
     op.value = i;
-    bitmap_->Bitfield(key_, {op}, &rets);
+    bitmap_->Bitfield(*ctx_, key_, {op}, &rets);
     EXPECT_EQ(1, rets.size());
     EXPECT_EQ(i - 1, rets[0].value());
     rets.clear();
+    ctx_->RefreshLatestSnapshot();
   }
-  auto s = bitmap_->Del(key_);
+  auto s = bitmap_->Del(*ctx_, key_);
 }
 
 TEST_P(RedisBitmapTest, SignedBitfieldWrapSetTest) {
@@ -362,25 +365,25 @@ TEST_P(RedisBitmapTest, SignedBitfieldWrapSetTest) {
   op.offset = 0;
   op.value = max;
 
-  bitmap_->Bitfield(key_, {op}, &rets);
+  bitmap_->Bitfield(*ctx_, key_, {op}, &rets);
   EXPECT_EQ(1, rets.size());
   EXPECT_EQ(0, rets[0].value());
   rets.clear();
 
   op.type = BitfieldOperation::Type::kIncrBy;
   op.value = 1;
-  bitmap_->Bitfield(key_, {op}, &rets);
+  bitmap_->Bitfield(*ctx_, key_, {op}, &rets);
   EXPECT_EQ(1, rets.size());
   EXPECT_EQ(min, rets[0].value());
   rets.clear();
 
   op.value = loopback;
-  bitmap_->Bitfield(key_, {op}, &rets);
+  bitmap_->Bitfield(*ctx_, key_, {op}, &rets);
   EXPECT_EQ(1, rets.size());
   EXPECT_EQ(min, rets[0].value());
   rets.clear();
 
-  auto s = bitmap_->Del(key_);
+  auto s = bitmap_->Del(*ctx_, key_);
 }
 
 TEST_P(RedisBitmapTest, UnsignedBitfieldWrapSetTest) {
@@ -397,25 +400,25 @@ TEST_P(RedisBitmapTest, UnsignedBitfieldWrapSetTest) {
   op.offset = 0;
   op.value = max;
 
-  bitmap_->Bitfield(key_, {op}, &rets);
+  bitmap_->Bitfield(*ctx_, key_, {op}, &rets);
   EXPECT_EQ(1, rets.size());
   EXPECT_EQ(0, rets[0].value());
   rets.clear();
 
   op.type = BitfieldOperation::Type::kIncrBy;
   op.value = 1;
-  bitmap_->Bitfield(key_, {op}, &rets);
+  bitmap_->Bitfield(*ctx_, key_, {op}, &rets);
   EXPECT_EQ(1, rets.size());
   EXPECT_EQ(0, rets[0].value());
   rets.clear();
 
   op.value = loopback;
-  bitmap_->Bitfield(key_, {op}, &rets);
+  bitmap_->Bitfield(*ctx_, key_, {op}, &rets);
   EXPECT_EQ(1, rets.size());
   EXPECT_EQ(0, rets[0].value());
   rets.clear();
 
-  auto s = bitmap_->Del(key_);
+  auto s = bitmap_->Del(*ctx_, key_);
 }
 
 TEST_P(RedisBitmapTest, SignedBitfieldSatSetTest) {
@@ -431,13 +434,13 @@ TEST_P(RedisBitmapTest, SignedBitfieldSatSetTest) {
   op.offset = 0;
   op.value = max * 2;
 
-  bitmap_->Bitfield(key_, {op}, &rets);
+  bitmap_->Bitfield(*ctx_, key_, {op}, &rets);
   EXPECT_EQ(1, rets.size());
   EXPECT_EQ(0, rets[0].value());
   rets.clear();
 
   op.type = BitfieldOperation::Type::kGet;
-  bitmap_->Bitfield(key_, {op}, &rets);
+  bitmap_->Bitfield(*ctx_, key_, {op}, &rets);
   EXPECT_EQ(1, rets.size());
   EXPECT_EQ(max, rets[0].value());
   rets.clear();
@@ -445,13 +448,14 @@ TEST_P(RedisBitmapTest, SignedBitfieldSatSetTest) {
   op.type = BitfieldOperation::Type::kIncrBy;
   for (int64_t i = 0; i <= max + 10; ++i) {
     op.value = i;
-    bitmap_->Bitfield(key_, {op}, &rets);
+    bitmap_->Bitfield(*ctx_, key_, {op}, &rets);
     EXPECT_EQ(1, rets.size());
     EXPECT_EQ(max, rets[0].value());
     rets.clear();
+    ctx_->RefreshLatestSnapshot();
   }
 
-  auto s = bitmap_->Del(key_);
+  auto s = bitmap_->Del(*ctx_, key_);
 }
 
 TEST_P(RedisBitmapTest, UnsignedBitfieldSatSetTest) {
@@ -468,13 +472,13 @@ TEST_P(RedisBitmapTest, UnsignedBitfieldSatSetTest) {
   op.offset = 0;
   op.value = max * 2;
 
-  bitmap_->Bitfield(key_, {op}, &rets);
+  bitmap_->Bitfield(*ctx_, key_, {op}, &rets);
   EXPECT_EQ(1, rets.size());
   EXPECT_EQ(0, rets[0].value());
   rets.clear();
 
   op.type = BitfieldOperation::Type::kGet;
-  bitmap_->Bitfield(key_, {op}, &rets);
+  bitmap_->Bitfield(*ctx_, key_, {op}, &rets);
   EXPECT_EQ(1, rets.size());
   EXPECT_EQ(max, rets[0].value());
   rets.clear();
@@ -482,13 +486,14 @@ TEST_P(RedisBitmapTest, UnsignedBitfieldSatSetTest) {
   op.type = BitfieldOperation::Type::kIncrBy;
   for (int64_t i = 0; i <= int64_t(max) + 10; ++i) {
     op.value = i;
-    bitmap_->Bitfield(key_, {op}, &rets);
+    bitmap_->Bitfield(*ctx_, key_, {op}, &rets);
     EXPECT_EQ(1, rets.size());
     EXPECT_EQ(max, rets[0].value());
     rets.clear();
+    ctx_->RefreshLatestSnapshot();
   }
 
-  auto s = bitmap_->Del(key_);
+  auto s = bitmap_->Del(*ctx_, key_);
 }
 
 TEST_P(RedisBitmapTest, SignedBitfieldFailSetTest) {
@@ -504,13 +509,13 @@ TEST_P(RedisBitmapTest, SignedBitfieldFailSetTest) {
   op.offset = 0;
   op.value = max * 2;
 
-  bitmap_->Bitfield(key_, {op}, &rets);
+  bitmap_->Bitfield(*ctx_, key_, {op}, &rets);
   EXPECT_EQ(1, rets.size());
   EXPECT_FALSE(rets[0].has_value());
   rets.clear();
 
   op.value = max;
-  bitmap_->Bitfield(key_, {op}, &rets);
+  bitmap_->Bitfield(*ctx_, key_, {op}, &rets);
   EXPECT_EQ(1, rets.size());
   EXPECT_EQ(0, rets[0].value());
   rets.clear();
@@ -518,13 +523,14 @@ TEST_P(RedisBitmapTest, SignedBitfieldFailSetTest) {
   op.type = BitfieldOperation::Type::kIncrBy;
   for (int64_t i = 1; i <= max; ++i) {
     op.value = i;
-    bitmap_->Bitfield(key_, {op}, &rets);
+    bitmap_->Bitfield(*ctx_, key_, {op}, &rets);
     EXPECT_EQ(1, rets.size());
     EXPECT_FALSE(rets[0].has_value());
     rets.clear();
+    ctx_->RefreshLatestSnapshot();
   }
 
-  auto s = bitmap_->Del(key_);
+  auto s = bitmap_->Del(*ctx_, key_);
 }
 
 TEST_P(RedisBitmapTest, UnsignedBitfieldFailSetTest) {
@@ -540,13 +546,13 @@ TEST_P(RedisBitmapTest, UnsignedBitfieldFailSetTest) {
   op.offset = 0;
   op.value = max * 2;
 
-  bitmap_->Bitfield(key_, {op}, &rets);
+  bitmap_->Bitfield(*ctx_, key_, {op}, &rets);
   EXPECT_EQ(1, rets.size());
   EXPECT_FALSE(rets[0].has_value());
   rets.clear();
 
   op.value = max;
-  bitmap_->Bitfield(key_, {op}, &rets);
+  bitmap_->Bitfield(*ctx_, key_, {op}, &rets);
   EXPECT_EQ(1, rets.size());
   EXPECT_EQ(0, rets[0].value());
   rets.clear();
@@ -554,13 +560,13 @@ TEST_P(RedisBitmapTest, UnsignedBitfieldFailSetTest) {
   op.type = BitfieldOperation::Type::kIncrBy;
   for (int64_t i = 1; i <= max; ++i) {
     op.value = i;
-    bitmap_->Bitfield(key_, {op}, &rets);
+    bitmap_->Bitfield(*ctx_, key_, {op}, &rets);
     EXPECT_EQ(1, rets.size());
     EXPECT_FALSE(rets[0].has_value());
     rets.clear();
   }
 
-  auto s = bitmap_->Del(key_);
+  auto s = bitmap_->Del(*ctx_, key_);
 }
 
 TEST_P(RedisBitmapTest, BitfieldStringGetSetTest) {
@@ -568,7 +574,7 @@ TEST_P(RedisBitmapTest, BitfieldStringGetSetTest) {
     GTEST_SKIP() << "skip bitmap test for BitfieldStringGetSetTest";
   }
   std::string str = "dan yuan ren chang jiu, qian li gong chan juan.";
-  string_->Set(key_, str);
+  string_->Set(*ctx_, key_, str);
 
   std::vector<std::optional<BitfieldValue>> rets;
 
@@ -579,7 +585,7 @@ TEST_P(RedisBitmapTest, BitfieldStringGetSetTest) {
   int i = 0;
   for (auto ch : str) {
     op.offset = i;
-    bitmap_->Bitfield(key_, {op}, &rets);
+    bitmap_->Bitfield(*ctx_, key_, {op}, &rets);
     EXPECT_EQ(1, rets.size());
     EXPECT_EQ(ch, rets[0].value());
     rets.clear();
@@ -588,7 +594,7 @@ TEST_P(RedisBitmapTest, BitfieldStringGetSetTest) {
 
   for (; static_cast<size_t>(i) <= str.size() + 10; i += 8) {
     op.offset = i;
-    bitmap_->Bitfield(key_, {op}, &rets);
+    bitmap_->Bitfield(*ctx_, key_, {op}, &rets);
     EXPECT_EQ(1, rets.size());
     EXPECT_EQ(0, rets[0].value());
     rets.clear();
@@ -599,14 +605,14 @@ TEST_P(RedisBitmapTest, BitfieldStringGetSetTest) {
   for (int l = 0, r = static_cast<int>(str.size() - 1); l < r; ++l, --r) {
     op.offset = l * 8;
     op.value = str[r];
-    bitmap_->Bitfield(key_, {op}, &rets);
+    bitmap_->Bitfield(*ctx_, key_, {op}, &rets);
     EXPECT_EQ(1, rets.size());
     EXPECT_EQ(str[l], rets[0].value());
     rets.clear();
 
     op.offset = r * 8;
     op.value = str[l];
-    bitmap_->Bitfield(key_, {op}, &rets);
+    bitmap_->Bitfield(*ctx_, key_, {op}, &rets);
     EXPECT_EQ(1, rets.size());
     EXPECT_EQ(str[r], rets[0].value());
     rets.clear();
@@ -618,7 +624,7 @@ TEST_P(RedisBitmapTest, BitfieldStringGetSetTest) {
   op.type = BitfieldOperation::Type::kGet;
   for (auto ch : str) {
     op.offset = i;
-    bitmap_->Bitfield(key_, {op}, &rets);
+    bitmap_->Bitfield(*ctx_, key_, {op}, &rets);
     EXPECT_EQ(1, rets.size());
     EXPECT_EQ(ch, rets[0].value());
     rets.clear();
