@@ -83,11 +83,19 @@ func (s *KvrocksServer) NewClient() *redis.Client {
 	return s.NewClientWithOption(&redis.Options{})
 }
 
+func optionsWithTimeouts(options *redis.Options) *redis.Options {
+	options.DialTimeout = 30 * time.Second
+	options.ReadTimeout = 30 * time.Second
+	options.WriteTimeout = 30 * time.Second
+	return options
+}
+
 func (s *KvrocksServer) NewClientWithOption(options *redis.Options) *redis.Client {
 	if options.Addr == "" {
 		options.Addr = s.addr.String()
 	}
-	return redis.NewClient(options)
+
+	return redis.NewClient(optionsWithTimeouts(options))
 }
 
 func (s *KvrocksServer) NewTCPClient() *TCPClient {
